package cage.audio;

import cage.core.Logger;
import cage.core.ResourceEngine;
import javazoom.jl.player.Player;

import java.io.BufferedInputStream;
import java.io.InputStream;

/**
 * The sound class represents a sound data file and contains a player for that file.
 * Currently the only format supported is MP3 via the JLayer library. Calling play will
 * create a thread which reads the file and sends it to the sound card. The player in this
 * thread is assigned to the player member field. If loop is true,
 * when the file ends it will be opened again and played again. Calling stop will close the
 * player and end the sound playing. If player is already assigned, that player is closed
 * when play is called. This limits a sound to only one currently playing player at once.
 * We did this so the script would not have to track instances of players.
 *
 * TODO:Removing this limit in some clean way should be a goal for future versions.
 *
 * @author Seth Marinello
 */
public class Sound {

    /** Set to true for looping in the play thread */
    private boolean loop = false;
    /** The name of the file the sound is loaded from */
    private String fileName = null;
    /** The resource name of the sound */
    private String name;
    /** The player for the sound */
    Player player;

    /**
     * Creates a Sound which will load from the specifed filename
     * @param fileName the file to load
     */
    public Sound(String fileName) {
        this.fileName = fileName;
    }

    /**
     * Creteas a sound with the specified resource name, fileName and looping option
     * @param name name of the sound
     * @param fileName the file to load
     * @param loop if the sound should loop or not
     */
    public Sound(String name, String fileName, boolean loop) {
                
        this.name = name;
        this.fileName = fileName;
        this.loop = loop;

        ResourceEngine.registerSound(name, this);
    }

    /**
     * Creates a new player and starts playback in a thread. If the Sound
     * is set to loop, a new player will be created when the sound finishes
     * and the sound will play again.
     */
    public void play() {
        new Thread() {
            public void run() {
                try {
                    do {
                        if (player != null) {
                            player.close();
                        }
                        player = createPlayer(fileName);
                        player.play();
                        while (!player.isComplete()) {
                        }
                        player.close();
                    } while (loop);
                }
                catch (Exception e) {
                    Logger.getRootLogger().error("Playing " + fileName + "failed because ", e);
                }
            }
        }.start();
    }

    /** If a player has been created, close the player. */
    public void stop() {
        if (player !=null) {
            player.close();
        }
    }

    /**
     * Returns the resource name of the sound
     * @return the sound name
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the name of the file this sound is loaded from
     * @return the filename
     */
    public String getFileName() {
        return fileName;
    }

    /**
     * Returns true if this sound will loop when played
     * @return should the sound loop
     * */
    public boolean getLoop() {
        return loop;
    }

    /** Sets if this sound should loop or not.
     * @param loop true if the sound should loop
     */
    public void setLoop(boolean loop) {
        this.loop = loop;
    }

    /** Helper method to create a player for play()
     * @param fileName the file to create the player for.
     * @return a player for the passed in audio file
     */
    private static Player createPlayer(String fileName) {
        Player ret = null;

        try {

            InputStream is = ResourceEngine.locateResourceAsStream(fileName);
            BufferedInputStream bis = new BufferedInputStream(is);
            ret = new Player(bis);
        }
        catch (Exception e) {
            Logger.getRootLogger().error("Problem playing file " + fileName + "because ", e);
        }

        return ret;
    }

    public String toString() {
        return  "(Sound) " + name + " file:" + fileName + " loop:" + loop;
    }
}
