package cage.carl;

import cage.core.Logger;
import cage.core.ResourceEngine;

import antlr.collections.AST;
import antlr.debug.misc.ASTFrame;

import java.io.*;
import java.util.HashMap;
import java.util.Set;

public class Carl
{
    /** The log */
    private static Logger log = Logger.getRootLogger();

    /** A map of all global CARL closures and variables */
    public static HashMap<String,Object> globals = new HashMap<String,Object>();

    public static void initializeBuiltins()
    {
        globals.putAll(Builtin.createBuiltins());
    }

    public static void loadScript(String scriptName)
    {
        InputStream scriptStream = ResourceEngine.locateResourceAsStream(scriptName);

        if (scriptStream == null)
        {
            log.error(scriptName + " was not loaded successfully");
        }
        else
        {
            BufferedInputStream buf = new BufferedInputStream(scriptStream);

            try
            {
                carlParser parser = new carlParser(new carlLexer(buf));

                parser.start();
                AST t = parser.getAST();

                if (t != null) {

                    if (log.getLevel() == Logger.Level.DEBUG) {
                        new ASTFrame("AST", t).setVisible(true);
                    }

                    carlTreeParser treeParser = new carlTreeParser();
                    treeParser.populateGlobals(globals);
                    treeParser.start(t);

                    globals.putAll(treeParser.getGlobals());
                }
                else {
                    log.error("Attempting to load CARL script " + scriptName + " produced nothing.");
                }
            }
            catch (antlr.TokenStreamException e) {
                log.error(e.getMessage(), e);
                System.exit(1);
            }
            catch (antlr.RecognitionException e) {
                log.error(e.getMessage(), e);
                System.exit(1);
            }
            catch (InvalidException e) {
                e.printStackTrace();
                log.error(e.getMessage(), e);
                System.exit(1);
            }
        }
    }

    synchronized public static void callAction(String actionName)
    {
        log.debug("Calling action " + actionName);

        if (globals == null) {
            log.error("Cannot call action " + actionName
                    + " because no scripts have been loaded succesfully");
            return;
        }

        Closure action = ((Closure)globals.get(actionName));

        if (action != null) {
            action.execute(globals);
        }
        else {
            log.error(actionName + " is not a defined action");
        }
    }

    public static Closure getAction(String actionName)
    {

        if (globals == null) {
            log.error("Cannot call action " + actionName
                    + " because no scripts have been loaded succesfully");
            return null;
        }

        Closure action = ((Closure)globals.get(actionName));

        if (action != null) {
            return action;
        }
        else {
            log.error(actionName + " is not a defined action");
            return null;
        }
    }

    public static String saveState()
    {
        String save = "<gamestate>\n";

        Set<String> keys = globals.keySet();

        for (String k : keys)
        {
            Object o = globals.get(k);
            if (o instanceof Integer)
            {
                save = save + "\t<var name=\"" + k + "\" type=\"int\" value=\"" + ((Integer)o) + "\"/>\n";
            }
            else if (o instanceof Double)
            {
                save = save + "\t<var name=\"" + k + "\" type=\"real\" value=\"" + ((Double)o) + "\"/>\n";
            }
            else if (o instanceof Boolean)
            {
                save = save + "\t<var name=\"" + k + "\" type=\"bool\" value=\"" + ((Boolean)o) + "\"/>\n";
            }
            else if (o instanceof Point)
            {
                save = save + "\t<var name=\"" + k + "\" type=\"point\" value=\"" + ((Point)o) + "\"/>\n";
            }
            else if (o instanceof String)
            {
                save = save + "\t<var name=\"" + k + "\" type=\"string\" value=\"" + ((String)o) + "\"/>\n";
            }
            // else it's a function/action or a 'nothing'
        }

        save = save + "</gamestate>\n";

        return save;
    }

    public static void restoreState(HashMap<String,Object> data)
    {
        globals.putAll(data);
    }
}
