package cage.core;

import cage.carl.Carl;
import cage.carl.Closure;
import cage.graphics.GraphicsEngine;
import cage.graphics.Scene;
import cage.input.MouseHandler;
import cage.ui.Overlay;
import org.minueto.MinuetoEventQueue;

import javax.swing.*;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLDecoder;
import java.util.Collection;


/**
 * Cage.java is the entry point into Cage. All production loading
 * and scene changes are handled by this class. Cage is the driver
 * for the separate Engines within the overall application. See documentation
 * on main method for parameters/args.
 * 
 *@author Seth Marinello
 *@date May 2007
 */
public class Cage {

    private static Logger log = Logger.getRootLogger();

    /** The length of the jar file extension */
    private final static int EXT_LENGTH = 4;

    /** The scene currently on display */
    private static Scene currentScene;
    /** The name of the production, defaulted to the engine name */
    private static String productionName = "Cage";
    /** The name of the author of this production */
    private static String productionAuthor = "Unknown";
    /** The first scene of the production */
    private static String productionStart = "start";

    /**
     * The queue where mouse/keyboard events are stored before being handled in
     * the timer loop.
     */
    private static MinuetoEventQueue queue = new MinuetoEventQueue();

    /**
     * This method sets up a production for running and displays it until
     * interuppted or input is given.
     * @param productionFile The production to load
     */
    public static void run(String productionFile) {

        log.info("Starting CAGE");

        log.info("Loading Script Builtins...");
        Carl.initializeBuiltins();

        // This will load the porduction and all related files
        log.info("Loading Configuration...");
        XMLEngine.parseProduction(XMLEngine.parseXML(productionFile));

        //Show the status of Resources
        log.info("Resource Report:");
        Logger.setIndent(3);
        ResourceEngine.printSounds();
        ResourceEngine.printVisualItems();
        Logger.setIndent(0);

        //Show all the scenes loaded for the production
        log.info("Scene Report:");
        Logger.setIndent(3);
        ResourceEngine.printScenes();
        Logger.setIndent(0);


        GraphicsEngine.setup(productionName + " by " + productionAuthor , true);

        changeScene(productionStart);

        GraphicsEngine.getDisplay().hideLoadingScreen();

        try {
            gameLoop(50);

        } catch (Throwable t) {
            log.error("A runtime error occured in Cage of type " + t.getClass().getName());
            if (log.getLevel() == Logger.Level.INFO || log.getLevel() == Logger.Level.DEBUG) {
                t.printStackTrace();
            }
            System.exit(1);
        }

    }

    /**
     * Sets the production name.
     * @param productionName The name
     */
    public static void setProductionName(String productionName) {
        Cage.productionName = productionName;
    }
    /**
     * Sets the production start.
     * @param productionStart The start
     */
    public static void setProductionStart(String productionStart) {
        Cage.productionStart = productionStart;
    }

    /**
     * Sets the production author.
     * @param productionAuthor The author
     */
    public static void setProductionAuthor(String productionAuthor) {
        Cage.productionAuthor = productionAuthor;
    }

    /**
     * Returns the scene currently displayed.
     * @return The current scene.
     */
    public static Scene getCurrentScene() {
        return currentScene;
    }

    /**
     * Returns the event queue for Cage input.
     * @return The event queue
     */
    public static MinuetoEventQueue getQueue() {
        return queue;
    }

    /**
     * Changes the scene by name lookup. If a scene is currently loaded, it
     * will remove the overlay of the previous scene as well.
     * @param sceneName The scene name to switch to.
     */
    public static void changeScene(String sceneName) {

        log.info("Changing Scene to " + sceneName);

        /* If a scene has been loaded before, remove its gui
         * Sprite graphics are cleared automatically 
         */
        if (currentScene != null) {

            if (currentScene.getOverlayName() != null) {
                log.info("Removing old Overlay");
                Overlay over = ResourceEngine.getOverlay(currentScene.getOverlayName());
                GraphicsEngine.removeOverlayComponents(over.getComponents());
            }
        }

        Scene newScene = ResourceEngine.getScene(sceneName);

        if (newScene == null) {
            log.error(sceneName + " is not the name of a defined scene, cannot change to it");
        }
        else {
            currentScene = newScene;

            //If the scene has an action, call that before displaying the scene
            if (currentScene.getStartActionName() != null) {
                Carl.callAction(currentScene.getStartActionName());
            }

            GraphicsEngine.setGraphics(currentScene.getGraphics());

            if (currentScene.getOverlayName() != null) {

                Overlay over = ResourceEngine.getOverlay(currentScene.getOverlayName());

                GraphicsEngine.addOverlayComponents(over.getComponents());

            }
        }

        //GraphicsEngine.getDisplay().hideLoadingScreen();
    }

    /**
     * Clears the currently loaded Scenes and loads a set of Scene, Carl game state values and
     * a saved scene location into CAGE, restoring the state from a file output by saveState().
     */
    public static void restoreState() {
        Collection<Scene> scenes = ResourceEngine.getAllScenes();

        //This will be the file we are saving to.
        String filePath = null;

        JFileChooser fc = new JFileChooser();
        int returnVal = fc.showOpenDialog(null);

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            log.info(file.getName() + " selected as the save file.");

            filePath = file.getAbsolutePath();

        } else {
            log.error("No save file selected");
            return;
        }

        ResourceEngine.clearScenes();
        XMLEngine.parseSave(XMLEngine.parseXML(filePath));

    }

    /**
     * Writes the current state of the game to an XML file.
     */
    public static void saveState() {
        Collection<Scene> scenes = ResourceEngine.getAllScenes();

        //This will be the file we are saving to.
        String filePath = null;

        JFileChooser fc = new JFileChooser();
        int returnVal = fc.showSaveDialog(null);

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            log.info(file.getName() + " selected as the save file.");

            filePath = file.getAbsolutePath();

        } else {
            log.error("No save file selected");
            return;
        }

        try {
            PrintWriter pw = new PrintWriter(new File(filePath));

            pw.write("<save location=\"" + currentScene.getName() + "\"" + ">\n");

            pw.write("<scenes>");
            for (Scene s : scenes) {
                pw.write(s.toXML());
            }
            pw.write("\n</scenes>\n");

            pw.write(Carl.saveState());

            pw.write("</save>\n");

            pw.flush();

            pw.close();

        } catch (IOException ex) {
            log.error("Could not save file " + filePath);
        }

    }

    /**
     * Starts CAGE running. CAGE accpets 5 different args. <br>
     * <ol>
     * <li>-d or --debug : puts the logger output at debug level
     * <li>-i or --info : puts the logger output at info level
     * <li>-e or --edit : puts the mouse handler in edit mode (right click to move sprites)
     * <li>-g or --gui : error level messages are displayed in GUI Dialogs
     * <li>other string : any other strings will be treated as a path to a production file,
     * with subsequent strings replacing the previous path.
     * </ol>
     * <br>
     * Some notes: by default the logger is set to error only reporting. If no file path
     * is specifed the program will assume it is being run in GUI mode and prompt for a
     * production file via an open dialog. All errors will be reported via dialogs in this mode.
     * <br>
     * The --gui option is provided for those who wish to pass a production file but still
     * retain the gui messages.
     * <br>
     * The operating system is also checked to see if we are running on an OS X machine.
     * If so the applcation name is set so the menu bar will display "CAGE" rather
     * than the classname, "cage.core.Cage".
     *
     */
    public static void main(String[] args) {
        String filePath = null;

        String osname = System.getProperty("os.name");
        //Sets the applcation name for the menu bar if running on a Mac
        if (osname.contains("Mac OS X")) {
            System.setProperty("com.apple.mrj.application.apple.menu.about.name", "CAGE");
        }

        for (String s : args) {
            if ("--debug".equalsIgnoreCase(s) || "-d".equalsIgnoreCase(s)) {
                Logger.getRootLogger().setLevel(Logger.Level.DEBUG);
            }
            else if ("--info".equalsIgnoreCase(s) || "-i".equalsIgnoreCase(s)) {
                Logger.getRootLogger().setLevel(Logger.Level.INFO);
            }
            else if ("--edit".equalsIgnoreCase(s) || "-e".equalsIgnoreCase(s)) {
                MouseHandler.editMode = true;
            }
            else if ("--gui".equalsIgnoreCase(s) || "-g".equalsIgnoreCase(s)) {
                Logger.guiMode = true;
            }
            else {
                filePath = s;
            }
        }

        if (filePath == null) {
            JFileChooser fc = new JFileChooser();
            int returnVal = fc.showOpenDialog(null);

            Logger.guiMode = true;

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                File file = fc.getSelectedFile();

                if (file == null) {
                    log.error("No production file selected");
                    System.exit(0);
                }
                else {
                    log.info(file.getName() + " selected as the production file.");
                }
                filePath = file.getAbsolutePath();

            } else {
                //The user hit cancel.
                System.exit(0);
            }
        }

        Cage.run(createProductionClasspath(filePath));
    }

    /**
     * Creates the path to be given to the URLClassLoader which will be used
     * to load the resources required by a production. The root is set to
     * the location of the production xml file, or the jar file containing
     * the production if a jar is given.
     * @param productionLocation the location of the production, either the XML file or the JAR
     * @return the path to the XML file to load as the production
     */
    private static String createProductionClasspath(String productionLocation) {
        String path = null;
        String fileName = null;
        int lastSlash = -1;

        URL ret = ResourceEngine.locateResource(productionLocation);

        try {
            path = URLDecoder.decode(ret.getPath(), "UTF-8");
        }
        catch (Exception e) {
            log.error("Bad URL Decoding in Production Path");    
        }


        if (path.contains("/")) {
            lastSlash = path.lastIndexOf('/');
        }
        else {
            lastSlash = path.lastIndexOf(File.separatorChar);
        }
        
        log.debug("Separator Character: " + File.separatorChar);
        log.debug("Intitial Production Location: " + productionLocation);
        log.debug("Path to Production: " + path);
        
        //If there were no slashes, the last one was at the start
        if (lastSlash < 0) {
            lastSlash = 0;
        }

        URL[] urls = new URL[1];
        //If the production is stored in a jar file we can use the actual file
        //the URL for the class loader
        if (productionLocation.endsWith(".jar") || productionLocation.endsWith(".zip")) {

            urls[0] = ResourceEngine.locateResource(productionLocation);

            //TODO more string bounds checking please
            String productionName = path.substring(lastSlash + 1, path.length() - EXT_LENGTH);

            fileName = productionName + ".xml";
        }
        else {   //Otherwise we have to parse the path out of the file path
            String pathWithoutFile = path.substring(0, lastSlash);

            //If there was no path before the filename
            if (pathWithoutFile.length() == 0) {
                pathWithoutFile = ".";
            }

            urls[0] = ResourceEngine.locateResource(pathWithoutFile + File.separator);
            fileName = productionLocation;
        }

        //The logger is not setup yet so this gets sent to the normal print method
        System.out.println("Production Locaiton = " + urls[0].getPath());

        URLClassLoader url = new URLClassLoader(urls);

        ResourceEngine.setResourceLoader(url);

        return fileName;
    }


    public static void gameLoop(int delay) {

        Collection<Closure> routines;

        //Added by Seth Marinello for CAGE
        //HashMap<Integer, KeyPressEvent> active = new HashMap<Integer, KeyPressEvent>();

        //LinkedList<Integer> pressedKeys = new LinkedList<Integer>();


        while(true) {

            try {
                Thread.yield();
                Thread.sleep(delay);
            } catch (InterruptedException ex) {
                log.error(ex.getMessage(), ex);
            }

            /*
            LinkedList events = (LinkedList)queue.getQueue().clone();

            pressedKeys.clear();

            for (Object o : events) {
                if (o instanceof KeyPressEvent) {
                    int val = ((KeyPressEvent)o).iValue;
                    //System.out.println("Putting " + val + " in pressed Keys");
                    pressedKeys.add(val);

                    if (!active.containsKey(((KeyPressEvent)o).iValue)) {
                        //System.out.println("Put a Key in Active with value " + val);
                        active.put(val, (KeyPressEvent)o);
                        KeyboardHandler.handleKeyPressReal(val);
                    }
                }
            }

            for (Object i : active.keySet().toArray()) {
                //If the key was active but is no longer giving input, it was released
                if (!pressedKeys.contains(i)) {
                    //System.out.println("Took a key out of active " + i);
                    KeyboardHandler.handleKeyReleaseReal((Integer)i);
                    active.remove(i);
                }
                else
                    ;
                //System.out.println("Pressed Keys has " + i);
            }
             */

            while(queue.hasNext()) {
                queue.handle();
            }

            //Scene rountines get to go first
            routines = currentScene.getRoutines();
            if (routines != null) {
                for(Closure c : routines) {
                    c.execute(Carl.globals);
                }
            }

            //Now global routines
            routines = ResourceEngine.getAllRountines();

            //Carl.globals was made public just for this, we can't afford an extra method call here.
            for(Closure c : routines) {
                c.execute(Carl.globals);
            }

            GraphicsEngine.render();
        }

    }
}


/**
 * Timer which encapsulates the game loop for Cage.
 * The event queue handling is placed here as well as
 * timing delay. The delay is in milliseconds. Done as a
 * separate thread so as to not interfere with CARL actions.

 private static class Timer extends Thread{

 int delay;
 Collection<Closure> routines;

 public Timer(int delay) {
 super();
 this.delay = delay;
 }

 public void run() {

 while(true) {

 try {

 sleep(delay);

 } catch (InterruptedException ex) {
 log.error(ex.getMessage(), ex);
 }

 while(queue.hasNext()) {
 queue.handle();
 }

 //Scene rountines get to go first
 routines = currentScene.getRoutines();
 for(Closure c : routines) {
 c.execute(Carl.globals);
 }

 //Now global routines
 routines = ResourceEngine.getAllRountines();

 //Carl.globals was made public just for this, we can't afford an extra method call here.
 for(Closure c : routines) {
 c.execute(Carl.globals);
 }

 GraphicsEngine.render();
 }
 }


 }
 */