package cage.core;

import javax.swing.*;
import java.io.PrintStream;

/**
 * This simple logging system is modeled after the fuctionality of log4j
 * but only includes the features needed for CAGE.
 *
 * A static logger named rootLogger can be access via <code>getRootLogger</code>.
 * The static logger by default prints to <code>System.out</code> and is at
 * ERROR level. By default, all loggers take the level and printing target from
 * the root logger. In general, classes can simply keep a reference to the rootLogger
 * and print to that, but instances of Logger can be created if you want to have
 * different levels of reporting between classes or if you want to have one class
 * print to a different stream than the root logger.
 *
 * The static message strings, <code>debugMessage</code>, <code>infoMessage</code>,
 * and <code>errorMessage</code> are prepended to all logger output of the respective
 * levels. The defaults can be replaced with custom strings via the setters.
 *
 * @author Seth Marinello
 * @date August 2007
 */
public class Logger {

    /**
     * Constants for the Level of the logger.
     * <ol>
     * <li>DEBUG - when at this level, the logger prints all messages
     * <li>INFO - Prints all but debug messages
     * <li>ERROR - Prints only error messages, for production release
     * <li>SILENT - No messages are printed, not recommended.
     * </ol>
     */
    public enum Level{DEBUG, INFO, ERROR, SILENT}

    /**
     * The PrintStream to print to. By default this is System.out.
     */
    private PrintStream out;

    /**
     * The output level of the logger.
     */
    private Level level;

    /**
     * The Amount of spaces to indent the output
     */
    private static int indent = 0;

    /**
     * If true, error level messages will be displayed in GUI Dialogs
     */
    public static boolean guiMode = false;

    /**
     * The spaces string created from the indent value
     */
    private static String indentString = "";

    private static String debugMessage = "Debug: ";
    private static String infoMessage = "Info: ";
    private static String errorMessage = "Error: ";

    private static Logger rootLogger = new Logger(System.out, Level.ERROR);

    /** Creates a default logger */
    public Logger() {
        out = rootLogger.getOut();
        level = rootLogger.getLevel();
    }

    /**
     * Creates a logger that uses the provided Printstream for output
     * @param out Stream to write the log to.
     */
    public Logger(PrintStream out) {
        this.out = out;
        level = rootLogger.getLevel();
    }

    /**
     * Creates a logger which the specified level but default print stream.
     * @param level Level to set the logger to.
     */
    public Logger(Level level) {
        out = rootLogger.getOut();
        this.level = level;
    }

    /**
     * Creates a logger with a custom print stream and level
     * @param out Stream to write the log to.
     * @param level Level to set the logger to.
     */
    public Logger(PrintStream out, Level level) {
        this.out = out;
        this.level = level;
    }

    /**
     * Print a debug message
     * @param obj Object to print
     */
    public void debug(Object obj) {
        if (level == Level.DEBUG) {
            out.println(indentString + debugMessage + obj);
        }
    }

    /**
     * Print a debug message
     * @param obj Object to print
     * @param t A throwable object, often the exception you are logging 
     */
    public void debug(Object obj, Throwable t) {
        if (level == Level.DEBUG) {
            out.println(indentString + debugMessage + obj + " " + t.getClass().getName());
        }
    }

    /**
     * Print a info message
     * @param obj Object to print
     */
    public void info(Object obj) {
        if (level == Level.INFO || level == Level.DEBUG) {
            out.println(indentString + infoMessage + obj);
        }
    }

    /**
     * Print a info message
     * @param obj Object to print
     * @param t A throwable object, often the exception you are logging
     */
    public void info(Object obj, Throwable t) {
        if (level == Level.INFO || level == Level.DEBUG) {
            out.println(indentString + infoMessage + obj + " " + t.getClass().getName());
        }
    }

    /**
     * Print a error message
     * @param obj Object to print
     */
    public void error(Object obj) {
        if (level != Level.SILENT) {

            String message = indentString + errorMessage + obj;

            if (guiMode) {
                createMessageDialog(message.trim());
            }
            out.println(message.trim());
        }
    }

    /**
     * Print a error message
     * @param obj Object to print
     * @param t A throwable object, often the exception you are logging
     */
    public void error(Object obj, Throwable t) {
        if (level != Level.SILENT) {

            String message = indentString + errorMessage + obj + " " + t.getClass().getName();

            if (guiMode) {
                createMessageDialog(message.trim());
            }
            out.println(message.trim());
        }
    }

    /**
     * Gets the message.
     * @return The message prepended to all debug messages.
     */
    public static String getDebugMessage() {
        return debugMessage;
    }

    /**
     * Sets the message.
     * @param debugMessage The message prepended to all debug messages.
     */
    public static void setDebugMessage(String debugMessage) {
        Logger.debugMessage = debugMessage;
    }

    /**
     * Gets the message.
     * @return The message prepended to all error messages.
     */
    public static String getErrorMessage() {
        return errorMessage;
    }

    /**
     * Sets the message.
     * @param errorMessage The message prepended to all error messages.
     */
    public static void setErrorMessage(String errorMessage) {
        Logger.errorMessage = errorMessage;
    }

    /**
     * Gets the message.
     * @return The message prepended to all debug messages.
     */
    public static String getInfoMessage() {
        return infoMessage;
    }

    /**
     * Sets the message.
     * @param infoMessage The message prepended to all info messages.
     */
    public static void setInfoMessage(String infoMessage) {
        Logger.infoMessage = infoMessage;
    }

    /**
     * The level of output allowed
     * @return the currentl level of the logger.
     */
    public Level getLevel() {
        return level;
    }

    /**
     * Sets the output level
     * @param level level to set this logger to
     */
    public void setLevel(Level level) {
        this.level = level;
    }

    /**
     * Returns the output stream
     * @return the PrintStream the logger prints to.
     */
    public PrintStream getOut() {
        return out;
    }

    /**
     * Sets the stream the logger writes to
     * @param out The stream to write to.
     */
    public void setOut(PrintStream out) {
        this.out = out;
    }

    /**
     * Gets the indent amount for all loggers.
     * @return indent amount in spaces
     */
    public static int getIndent() {
        return indent;
    }

    /**
     * Sets the indent amount in spaces.
     * @param indent number of spaces to indent
     */
    public static void setIndent(int indent) {
        Logger.indent = indent;
        createIndent();
    }

    /**
     * Gets the static logger all loggers grow from.
     * @return  the root logger
     */
    public static Logger getRootLogger() {
        return rootLogger;
    }

    /**
     * Allows replacement of the root logger
     * @param rootLogger logger to replace the root logger.
     */
    public static void setRootLogger(Logger rootLogger) {
        Logger.rootLogger = rootLogger;
    }

    /**
     * Private helper method to create the indednt string
     */
    private static void createIndent() {

        indentString = "";

        for (int i = 0; i < indent; i++) {
            indentString += " ";
        }

    }

    private void createMessageDialog(String message) {
        JOptionPane.showMessageDialog(null, message, "CAGE Error",
                JOptionPane.ERROR_MESSAGE);
    }
}
