package cage.core;

import cage.audio.Sound;
import cage.core.Logger;
import cage.graphics.VisualItem;
import cage.graphics.Scene;
import cage.input.Binding;
import cage.carl.Closure;
import cage.carl.Carl;
import cage.ui.Overlay;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 * Cage and Carl
 *
 * Resource Engine
 *
 * This class contains static maps for each datatype used within the engine.
 * All data classes (Overlay, VisualItem, Sound, etc) should call register[TYPE]
 * within their constructors so that they are always registered when created.
 * A notable exception to this is TextImage, which is intended to be temporary
 * and so it is not registered by default.
 * This is important because the maps here act as a bridge between the XML
 * scene definitions, the Carl scripts and the actual engine methods and data.
 *
 * @author Seth Marinello
 * @date May 12, 2007
 */
public class ResourceEngine {

    /** The log. */
    private static Logger log = Logger.getRootLogger();

    /** The overlays. */
    private static Map<String, Overlay> overlays = new HashMap<String, Overlay>();

    /** The scenes. */
    private static Map<String, Scene> scenes = new HashMap<String, Scene>();

    /** The images. */
    private static Map<String, VisualItem> images = new HashMap<String, VisualItem>();

    /** The sounds. */
    private static Map<String, Sound> sounds = new HashMap<String, Sound>();

    /** The bindings */
    private static Map<Integer, Binding> bindings = new HashMap<Integer, Binding>();

    /** The routines */
    private static Map<String, Closure> routines = new HashMap<String, Closure>();



    /**
     * This is a class loader pointing to the location where the production file was defined.
     * ResourceEngine's file location methods use this to create a custom Classpath at runtime
     * relative to the location of the active presentation.
     */
    private static ClassLoader resourceLoader = ResourceEngine.class.getClassLoader();


    /**
     * Registers an VisualItem for use by different sprites in the Graphics Engine.
     *
     * @param name The name to use as a key in the VisualItem map
     * @param img The VisualItem data to store with the key
     */
    public static void registerVisualItem(String name, VisualItem img) {
        images.put(name, img);
    }

    /**
     * Gets the visual item.
     *
     * @param key the key
     *
     * @return the visual item
     */
    public static VisualItem getVisualItem(String key) {
        return images.get(key);
    }


    /**
     * Gets the all visual items.
     *
     * @return the all visual items
     */
    public static Collection<VisualItem> getAllVisualItems() {
        return images.values();
    }

    /**
     * Unregister visual item.
     *
     * @param key the key
     */
    public static void unregisterVisualItem(String key) {
        images.remove(key);
    }

    /**
     * Register sound.
     *
     * @param name the name
     * @param snd the snd
     */
    public static void registerSound(String name, Sound snd) {
        sounds.put(name, snd);
    }

    /**
     * Gets the sound.
     *
     * @param key the key
     *
     * @return the sound
     */
    public static Sound getSound(String key) {
        return sounds.get(key);
    }

    /**
     * Unregister sound.
     *
     * @param key the key
     */
    public static void unregisterSound(String key) {
        sounds.remove(key);
    }

    /**
     * Register scene.
     *
     * @param s the s
     */
    public static void registerScene(Scene s) {
        scenes.put(s.getName(), s);
    }

    /**
     * Unregister scene.
     *
     * @param key the key
     *
     * @return the scene
     */
    public static Scene unregisterScene(String key) {
        return scenes.remove(key);
    }

    /**
     * Register overlay.
     *
     * @param s the s
     */
    public static void registerOverlay(Overlay s) {
        overlays.put(s.getName(), s);
    }

    /**
     * Unregister overlay.
     *
     * @param key the key
     *
     * @return the overlay
     */
    public static Overlay unregisterOverlay(String key) {
        return overlays.remove(key);
    }

    /**
     * Gets the overlay.
     *
     * @param key the key
     *
     * @return the overlay
     */
    public static Overlay getOverlay(String key) {
        return overlays.get(key);
    }

    /**
     * Get all Overlays
     * @return A collection of all Overlays.
     */
    public static Collection<Overlay> getAllOverlays() {
        return overlays.values();
    }

    /**
     * Gets the scene.
     *
     * @param key the key
     *
     * @return the scene
     */
    public static Scene getScene(String key) {
        return scenes.get(key);
    }

    /**
     * Gets the all scenes.
     *
     * @return the all scenes
     */
    public static Collection<Scene> getAllScenes() {
        return scenes.values();
    }

    /**
     * Removes/Deletes all loaded scenes.
     */
    public static void clearScenes() {
        //Remove all the scenes
        scenes.clear();
        //Encourage Java to garbage collect
        System.gc();
    }

    /**
     * Adds a binding to the global bindings
     * @param keyConst the value of the key the bind is for
     * @param bind the Binding for the key value.
     */
    public static void registerBinding(Integer keyConst, Binding bind) {
        bindings.put(keyConst, bind);
    }

    /**
     * Removes a binding from the global bindings
     * @param keyConst the value of the key the bind is for
     */
    public static void unregisterBinding(Integer keyConst) {
        bindings.remove(keyConst);
    }

    /**
     * Get a binding from the global bindings.
     * @param keyConst the KeyValue of the binding desired.
     * @return The Binding object for that key or null.
     */
    public static Binding getBinding(Integer keyConst) {
        return bindings.get(keyConst);
    }

    /**
     * Adds an action to a list of actions which will be called each tick of the engine clock
     * @param name The name of the action to add
     */
    public static void registerRountine(String name) {
        Closure action = Carl.getAction(name);

        if (action != null) {
            routines.put(name, action);
        }
    }

    /**
     * Removes an action from a list of actions which will be called each tick of the engine clock
     * @param name The name of the action to remove
     */
    public static void unregisterRoutine(String name) {
        routines.remove(name);
    }

    public static Collection<Closure> getAllRountines() {
        return routines.values();
    }

    /* ---------- Print Methods ---------- */

    /**
     * Prints the overlays.
     */
    public static void printOverlays() {
        for (Overlay s : overlays.values()) {
            log.info(s);
        }
    }

    /**
     * Prints the scenes.
     */
    public static void printScenes() {
        for (Scene s : scenes.values()) {
            log.info(s);
        }
    }

    /**
     * Prints the sounds.
     */
    public static void printSounds() {
        for (Sound s : sounds.values()) {
            log.info(s);
        }
    }

    /**
     * Prints the visual items.
     */
    public static void printVisualItems() {
        for (VisualItem s : images.values()) {
            log.info(s);
        }
    }

    /**
     * Sets the ClassLoader to be used by the ResourceEngine when locating resources
     * @param resourceLoader The custom ClassLoader to use.
     */
    public static void setResourceLoader(ClassLoader resourceLoader) {
        ResourceEngine.resourceLoader = resourceLoader;
    }

    /**
     * Locate resource.
     *
     * @param filename the filename
     *
     * @return the uRL
     */
    public static URL locateResource(String filename) {
        URL ret;

        ret = resourceLoader.getResource(filename);


        if (ret == null) {
            try {
                ret = new File(filename).toURI().toURL();
            } catch (IOException ex) {
                log.error("The file " + filename + " could not be found. Remember, file names" +
                        " can be both case and spelling sensitive.");
            }

        }

        return ret;
    }

    /**
     * Locate resource as stream.
     *
     * @param filename the filename
     *
     * @return the input stream
     */
    public static InputStream locateResourceAsStream(String filename) {
        InputStream ret;

        ret = resourceLoader.getResourceAsStream(filename);

        if (ret == null) {
            try {
                ret = new FileInputStream(filename);
            } catch (IOException ex) {
                log.error("The file " + filename + " could not be found. Remember, file names" +
                        " can be both case and spelling sensitive.");
            }

        }

        return ret;
    }

}
