package cage.core;

import cage.carl.Carl;
import cage.ui.Overlay;
import cage.graphics.Scene;
import cage.core.ResourceEngine;
import cage.graphics.*;
import cage.graphics.Image;
import cage.graphics.Point;
import cage.audio.Sound;
import cage.ui.GUIFactory;
import cage.input.Binding;
import nanoxml.XMLElement;
import nanoxml.XMLParseException;

import java.awt.Color;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.InputStream;
import java.util.Vector;
import java.util.HashMap;

import org.minueto.handlers.MinuetoKeyboard;

/**
 * XMLEngine is a collection of methods for parsing various aspects of
 * the XML files used in CAGE.
 *
 * @author Seth Marinello
 * @date August 2007
 */
public class XMLEngine {

    /** The log. */
    private static Logger log = Logger.getRootLogger();

    /**
     * Loads the XML file and parses it into an XML document tree.
     *
     * @param filename filename to load
     *
     * @return The root element of the tree
     */
    public static XMLElement parseXML(String filename) {
        XMLElement doc = new XMLElement();


        try {

            InputStream is = ResourceEngine.locateResourceAsStream(filename);

            InputStreamReader reader = new InputStreamReader(is);
            doc.parseFromReader(reader);

        } catch (IOException ex2) {
            log.error("XML Document:" + filename + " could not be found or read!");
            System.exit(1);
        } catch (XMLParseException xmlEX) {
            log.error("Could not parse XML for " + filename + " because of " + xmlEX.getMessage());
            System.exit(1);
        } catch (NullPointerException ex) {
            log.error("XML Document:" + filename + " was null!");
            System.exit(1);
        }


        return doc;
    }

    /**
     * Parse a tree defining resources.
     *
     * @param element root of resources tree
     */
    public static void parseResources(XMLElement element) {

        Vector elements = element.getChildren();

        for (Object element1 : elements) {
            XMLElement child = (XMLElement) element1;
            if (child.getName().equals("image")) {
                extractImage(child).getName();
            } else if (child.getName().equals("sound")) {
                extractSound(child).getName();
            } else if (child.getName().equals("animation")) {
                extractAnimation(child).getName();
            }
        }
    }

    /**
     * Creates an Image from an XMLElement.
     *
     * @param element element defining the Image
     *
     * @return the Image created
     */
    public static Image extractImage(XMLElement element) {

        String name = (String)element.getAttribute("name");
        String file = (String)element.getAttribute("src");
        try {
            return new Image(name, file);
        } catch (Exception e)  {
            log.error("Loading Image " + name + " from " + file + " failed.");
        }

        return null;
    }

    /**
     * Creates an Animation from an XMLElement.
     *
     * @param element element defining the Animation
     *
     * @return the Animation created
     */
    public static Animation extractAnimation(XMLElement element) {
        Animation ret = null;

        String name = (String)element.getAttribute("name");
        String file = (String)element.getAttribute("src");

        try {

            ret = new Animation(name, file);

        } catch (Exception e)  {
            log.error("Loading Image " + name + " from " + file + " failed.");
        }

        return ret;
    }

    /**
     * Creates a Sound from an XMLElement.
     *
     * @param element element defining the Sound
     *
     * @return the Sound created
     */
    public static Sound extractSound(XMLElement element) {

        String name = (String)element.getAttribute("name");
        String file = (String)element.getAttribute("src");
        String value = (String)element.getAttribute("loop");
        boolean loop = "true".equalsIgnoreCase(value);

        try {
            return new Sound(name, file, loop);
        } catch (Exception e)  {
            log.error("Loading Image " + name + " from " + file + " failed.");
        }

        return null;
    }

    public static void parseSave(XMLElement element) {
        Vector<XMLElement> elements = element.getChildren();

        String locationName = (String)element.getAttribute("location");
        locationName = locationName == null ? "start" : locationName ;

        for (int i = 0; i < elements.size(); i++) {
            XMLElement child = elements.get(i);
            if (child.getName().equals("scenes")) {
                parseScenes(child);
            }
            else if (child.getName().equals("gamadata")) {
                parseGamedata(child);
            }
        }

        Cage.changeScene(locationName);
    }

    public static void extractGamedata(XMLElement element, HashMap<String,Object> vars) {
        String name = (String)element.getAttribute("name");
        String type = (String)element.getAttribute("type");
        String value = (String)element.getAttribute("value");

        if (type.equals("int")) {
            vars.put(name, new Integer(value));
        }
        else if (type.equals("real")) {
            vars.put(name, new Double(value));
        }
        else if (type.equals("bool")) {
            vars.put(name, new Boolean(value));
        }
        else if (type.equals("point")) {
            vars.put(name, new cage.carl.Point(value));
        }
        else if (type.equals("string")) {
            vars.put(name, value);
        }
    }

    public static void parseGamedata(XMLElement element) {
        Vector<XMLElement> elements = element.getChildren();

        HashMap<String,Object> vars = new HashMap<String,Object>();

        for (int i = 0; i < elements.size(); i++) {
            XMLElement child = elements.get(i);
            if (child.getName().equals("var")) {
                extractGamedata(child, vars);
            }
        }

        Carl.restoreState(vars);
    }

    /**
     * Parses the scenes.
     *
     * @param element the element
     */
    public static void parseScenes(XMLElement element) {

        Vector<XMLElement> elements = element.getChildren();

        for (int i = 0; i < elements.size(); i++) {
            XMLElement child = elements.get(i);
            if (child.getName().equals("scene")) {
                extractScene(child).getName();
            }
            else if (child.getName().equals("overlay")) {
                extractOverlay(child).getName();
            }
        }
    }

    /**
     * Extract scene.
     *
     * @param element the element
     *
     * @return the scene
     */
    public static Scene extractScene(XMLElement element) {

        String name = (String)element.getAttribute("name");
        Scene ret  = new Scene(name);

        String oName = (String)element.getAttribute("overlay");

        ret.setOverlayName(oName);

        String startAction = (String)element.getAttribute("action");

        ret.setStartActionName(startAction);

        Vector<XMLElement> elements = element.getChildren();

        XMLElement child;

        for (int i = 0; i < elements.size(); i++) {
            child = elements.get(i);

            if (child.getName().equals("sprite")) {
                ret.addSprite(extractSprite(child));
            }
            else if (child.getName().equals("binding")) {
                Binding b = extractBinding(child);
                ret.addBinding(b.keyCode, b);
            }
            else if (child.getName().equalsIgnoreCase(("routine"))) {
                ret.addRoutine((String)child.getAttribute("action"));
            }
        }
        return ret;

    }

    /**
     * Extract sprite.
     *
     * @param element the element
     *
     * @return the sprite
     */
    public static Sprite extractSprite(XMLElement element) {

        Sprite ret;

        String filename = (String)element.getAttribute("src");
        String name = (String)element.getAttribute("name");

        String value = (String)element.getAttribute("x");
        int x = value == null ? 0 : Integer.parseInt(value);

        value = (String)element.getAttribute("y");
        int y = value == null ? 0 : Integer.parseInt(value);

        ret = new Sprite(filename, name, new Point(x,y));

        value = (String)element.getAttribute("visible");

        if (value != null && value.equalsIgnoreCase("false")) {
            ret.setVisible(false);
        }

        value = (String)element.getAttribute("action");
        ret.setActionName(value);

        value = (String)element.getAttribute("loop");
        boolean loop = "true".equalsIgnoreCase(value);

        value = (String)element.getAttribute("play");
        boolean play = "true".equalsIgnoreCase(value);

        ret.setLoop(loop);
        ret.setPlay(play);

        //FIND THE HOT SPOTS
        Vector<XMLElement> elements = element.getChildren();

        XMLElement child;

        for (int i = 0; i < elements.size(); i++) {
            child = elements.get(i);

            if (child.getName().equals("hotspot")) {
                ret.addHotSpot(extractHotSpot(child, x, y));
            }
        }


        return ret;
    }

    /**
     * Extract hot spot.
     *
     * @param element the element
     *
     * @return the hot spot
     */
    public static HotSpot extractHotSpot(XMLElement element, int xOff, int yOff) {

        String name = (String)element.getAttribute("name");

        String value = (String)element.getAttribute("sizeX");
        int sizeX = value == null ? 20 : Integer.parseInt(value);

        value = (String)element.getAttribute("sizeY");
        int sizeY = value == null ? 20 : Integer.parseInt(value);

        value = (String)element.getAttribute("X");
        int X = value == null ? xOff : Integer.parseInt(value) + xOff;

        value = (String)element.getAttribute("Y");
        int Y = value == null ? yOff : Integer.parseInt(value) + yOff;

        value = (String)element.getAttribute("active");
        boolean active = value == null ? true : "true".equalsIgnoreCase(value);

        value = (String)element.getAttribute("action");

        return new HotSpot(name, value, X, Y, sizeX, sizeY, active);
    }

    /**
     * Parses the production.
     *
     * @param element the element
     */
    public static void parseProduction(XMLElement element) {


        String name = (String)element.getAttribute("name");
        String author = (String)element.getAttribute("author");
        String start = (String)element.getAttribute("start");

        if (name != null) {
            Cage.setProductionName(name);
        }

        if (author != null) {
            Cage.setProductionAuthor(author);
        }

        if (start != null)  {
            Cage.setProductionStart(start);
        }

        Vector<XMLElement> elements = element.getChildren();

        for (int i = 0; i < elements.size(); i++) {
            XMLElement child = elements.get(i);

            if (child.getName().equalsIgnoreCase("width")) {
                int xNat, xAct;

                //TODO MORE ERROR CHECKING
                xNat = Integer.parseInt((String)child.getAttribute("natural"));
                xAct = Integer.parseInt((String)child.getAttribute("actual"));

                GraphicsEngine.setWidth(xNat);

                if (xAct != xNat) {
                    GraphicsEngine.setXScale((double)xNat/(double)xAct);
                }

            }
            else if (child.getName().equalsIgnoreCase("height")) {
                int yNat, yAct;

                //TODO MORE ERROR CHECKING
                yNat = Integer.parseInt((String)child.getAttribute("natural"));
                yAct = Integer.parseInt((String)child.getAttribute("actual"));

                GraphicsEngine.setHeight(yNat);

                if (yAct != yNat) {
                    GraphicsEngine.setYScale((double)yNat/(double)yAct);
                }

            }
            //else if (child.getName().equals("font")) {
            //    extractFont(child).getName();
            //}
            else if (child.getName().equalsIgnoreCase("resources")){
                log.info("Loading Resources from " + child.getAttribute("src"));
                parseResources(parseXML((String)child.getAttribute("src")));
            }
            else if (child.getName().equalsIgnoreCase("scripts")){
                log.info("Loading Script " + child.getAttribute("src"));
                Logger.setIndent(3);
                Carl.loadScript((String)child.getAttribute("src"));
                Logger.setIndent(0);
            }
            else if (child.getName().equalsIgnoreCase("scenes")){
                log.info("Loading Scenes from " + child.getAttribute("src"));
                parseScenes(parseXML((String)child.getAttribute("src")));
            }
            else if (child.getName().equalsIgnoreCase(("binding"))) {
                Binding b = extractBinding(child);
                ResourceEngine.registerBinding(b.keyCode, b);
            }
            else if (child.getName().equalsIgnoreCase(("routine"))) {
                ResourceEngine.registerRountine((String)child.getAttribute("action"));
            }
        }

    }

    public static Binding extractBinding(XMLElement element) {

        Binding b = new Binding();

        String value = (String)element.getAttribute("key");

        b.keyName = value;

        int keyValue = -1;
        try {
            keyValue = MinuetoKeyboard.class.getDeclaredField(value).getInt(null);

        } catch (NoSuchFieldException ex) {
            log.error("Could not find key " + value);
        } catch (IllegalAccessException ex) {
            log.error("Could not get value for key " + value);
        }

        value = (String)element.getAttribute("action");

        b.keyCode = keyValue;
        b.actionName = value;

        return b;
    }

    /**
     * Extract overlay.
     *
     * @param element the element
     *
     * @return the overlay
     */
    public static Overlay extractOverlay(XMLElement element) {

        String name = (String)element.getAttribute("name");
        Overlay ret  = new Overlay(name);

        Vector<XMLElement> elements = element.getChildren();

        for (XMLElement child : elements) {

            if (child.getName().equals("TextArea")) {
                ret.addComponent(extractTextArea(child));
            }
        }
        return ret;
    }

    /**
     * Extract text area.
     *
     * @param element the element
     *
     * @return the overlay. named panel
     */
    public static Overlay.NamedPanel extractTextArea(XMLElement element) {

        String name = (String)element.getAttribute("name");

        String text = (String)element.getAttribute("text");

        String value = (String)element.getAttribute("sizeX");
        int sizeX = value == null ? 20 : Integer.parseInt(value);

        value = (String)element.getAttribute("sizeY");
        int sizeY = value == null ? 20 : Integer.parseInt(value);

        value = (String)element.getAttribute("X");
        int X = value == null ? 0 : Integer.parseInt(value);

        value = (String)element.getAttribute("Y");
        int Y = value == null ? 0 : Integer.parseInt(value);

        value = (String)element.getAttribute("editable");
        boolean editable = value == null ? false : value.equalsIgnoreCase("true");

        value = (String)element.getAttribute("italic");
        boolean italic = value == null ? false : value.equalsIgnoreCase("true");

        return  GUIFactory.createColoredTextArea(name, text, Color.WHITE,  Color.BLACK,
                new java.awt.Point(X, Y),
                new java.awt.Point(sizeX, sizeY), editable);
    }

}
