package cage.graphics;

import cage.core.ResourceEngine;
import cage.core.Logger;

import javax.imageio.stream.ImageInputStream;
import javax.imageio.ImageReader;
import javax.imageio.ImageIO;
import java.util.Iterator;
import java.util.ArrayList;
import java.io.IOException;
import java.awt.image.BufferedImage;

import org.minueto.image.MinuetoImage;


/**
 * An animation is an Image with several frames. The first frame of the animation will be
 * used as the master frame, defining the size, etc. Individual frames can be accessed by
 * the getFrame method. Sprites support Animations through the VisualItem interface.
 *
 * @author Seth Marinello
 */
public class Animation implements VisualItem {

    /** The frames. */
    private ArrayList<MinuetoImage> frames;

    /** The name. */
    private String name;

    /** The filename. */
    private String filename;

    /**
     * Instantiates a new animation.
     * 
     * @param name the name
     * @param filename the filename
     */
    public Animation(String name, String filename) {

        this.name = name;
        this.filename = filename;

        frames = new ArrayList<MinuetoImage>();
        try {
            Object input = ResourceEngine.locateResourceAsStream(filename);
            ImageInputStream stream = ImageIO.createImageInputStream(input);
            Iterator readers = ImageIO.getImageReaders(stream);

            if (!readers.hasNext())
                throw new RuntimeException("no image reader found.");

            ImageReader reader = (ImageReader) readers.next();

            reader.setInput(stream);
            int n = reader.getNumImages(true);

            //System.out.println("numImages = " + n);
            for (int i = 0; i < n; i++) {
                BufferedImage image = reader.read(i);

                frames.add(new MinuetoImage(image));
            }
            stream.close();

            ResourceEngine.registerVisualItem(name,this);

        } catch (IOException ex) {
            Logger.getRootLogger().error("Animation failed to load from " + filename);
        }
    }

    /* (non-Javadoc)
     * @see cage.graphics.VisualItem#scale(double, double)
     */
    public void scale(double xScale, double yScale) {
        for (int i = 0; i < frames.size() ; i++) {
             frames.set(i, frames.get(i).scale(xScale, yScale));
        }
    }

    /**
     * Gets the frame.
     * 
     * @param index the index
     * 
     * @return the frame
     */
    public MinuetoImage getFrame(int index) {
        return frames.get(index);
    }

    /**
     * Gets the frame count.
     * 
     * @return the frame count
     */
    public int getFrameCount() {
        return frames.size();
    }

    /* (non-Javadoc)
     * @see cage.graphics.VisualItem#getName()
     */
    public String getName() {
        return name;
    }

    /* (non-Javadoc)
     * @see cage.graphics.VisualItem#getImage()
     */
    public MinuetoImage getImage() {
        return frames.get(0);
    }

    /* (non-Javadoc)
     * @see cage.graphics.VisualItem#getWidth()
     */
    public int getWidth() {
        return frames.get(0).getWidth();
    }

    /* (non-Javadoc)
     * @see cage.graphics.VisualItem#getHeight()
     */
    public int getHeight() {
        return frames.get(0).getHeight();
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return  "(Animation) " + name + " file:" + filename;
    }
}



