package cage.graphics;

import cage.core.Cage;
import cage.core.Logger;
import cage.carl.Carl;

/**
 * An abstract class which contains collision detection code
 * with mouse input, for use with anything which has a
 * rectangular hitbox and is intended to be clicked.
 *
 * @author Seth Marinello
 */
public abstract class Clickable {

    /**
     * The top left of the bounding box on this Clickable
     */
    protected Point topLeft = null;

    /**
     * The bottom right of the bounding box on this Clickable
     */
    protected Point botRight = null;

    /** The width of this clickable's hit box. */
    protected int width;

    /** The height of this clickable's hit box */
    protected int height;

    /**
     * The name of the action to call when clicked.
     */
    protected String actionName;

    /**
     * The name of the clickable item
     */
    protected String name;

    /**
     * Checks to see if the given input coords are within the
     * bounding box of this clickable.
     * @param x the x value to test
     * @param y the y value to test
     * @return Returns true if the given coordinate is within this
     * Clickables bound box.
     */
    public boolean inputCollision(int x, int y) {

        return (x >= topLeft.getX() && x <= botRight.getX()
                && y >= topLeft.getY() && y <= botRight.getY());

    }


    /**
     * Gets the X,Y location of this Sprite.
     *
     * @return the X,Y location as a Point
     */
    public Point getLocation() {
        return topLeft;
    }

    /**
     * Set the location of this sprite to the specified point.
     *
     * @param coord the X,Y location of the sprite
     */
    public void setLocation(Point coord) {
        setLocation(coord.getX(), coord.getY());
    }

    /**
     * Set the location of this Clickable to the specified X,Y.
     *
     * @param x the x coord
     * @param y the y coord
     */
    public void setLocation(int x, int y) {
        int xShift = x - topLeft.getX();
        int yShift = y - topLeft.getY();

        Logger.getRootLogger().debug("X Size:" + xShift + " Y size " + yShift );

        this.topLeft.setX(x);
        this.topLeft.setY(y);
        this.botRight.setX(topLeft.getX() + width);
        this.botRight.setY(topLeft.getY() + height);

    }

    /**
     * Get the X location of the upper left corner
     * of this clickable.
     * @return the top left x value.
     */
    public int getX() {
        return topLeft.getX();
    }

    /**
     * Get the Y location of the upper left corner
     * of this clickable.
     * @return the top left y value.
     */
    public int getY() {
        return topLeft.getY();
    }

    /**
     * Perform the action of this clickable, for use with inputCollision 
     */
    public void performAction() {
        if (actionName != null) {
            Carl.callAction(actionName);
        }
    }

    /**
     * Get the name of this Clickable.
     * @return tbe name.
     */
    public String getName(){
        return name;
    }
}
