package cage.graphics;

import org.minueto.window.MinuetoWindow;
import org.minueto.window.MinuetoPanel;

import javax.swing.*;
import java.awt.*;
import java.util.LinkedList;

/**
 * A layered window which wraps a number of Components and a Canvas
 * which can be rendered to into one object. This allows for the mixture
 * of AWT/Swing Components and the drawing surface provided by the
 * Minueto library. The window will automatically center itself upon
 * creation.
 *
 * @author Seth Marinello
 */
public class Display extends JFrame {

    private JLayeredPane content;

    private  MinuetoPanel graphicsPanel;

    private  LinkedList<Component> addedItems;

    private Panel loadingPanel;

    /**
     * This creates a display window holding a panel to render graphics to
     * as well as layers for holding overlays.
     * @param width the width of the display
     * @param height the height of the display
     * @param border if the display window should draw the window border
     */
    public Display(int width, int height, boolean border) {

        setSize(width, height);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setResizable(false);
        setUndecorated(!border);

        addedItems = new LinkedList<Component>();

        graphicsPanel = new MinuetoPanel(width, height);

        Panel graphicsHolder = graphicsPanel.getPanel();

        loadingPanel = new Panel();
        loadingPanel.setBackground(java.awt.Color.BLACK);
        loadingPanel.setBounds(0, 0, width, height);

        content = new JLayeredPane();
        content.setSize(width, height);
        content.setPreferredSize(content.getSize());

        content.add(graphicsHolder, JLayeredPane.PALETTE_LAYER);

        getContentPane().add(content);

        showLoadingScreen();

        setVisible(true);

        pack();
        setLocationRelativeTo(null);
        graphicsPanel.enable();


    }

    /**
     * Places a blank screen over the whole display, for use when loading
     */
    public void showLoadingScreen() {
        content.add(loadingPanel, JLayeredPane.POPUP_LAYER);
    }

    /**
     * Removes the loading screen from the display.
     */
    public void hideLoadingScreen() {
        content.remove(loadingPanel);
    }

    /**
     * Provides access to the panel which is rendered to by the GraphicsEngine
     * @return the canvas to render on
     */
    public MinuetoWindow getGraphicsDisplay() {
        return graphicsPanel;
    }

    /**
     * Remove all compoents that have been added to the Display since
     * it was created.
     */
    public void removeGui() {
        for (Component c : addedItems) {
            content.remove(c);
        }

        addedItems.clear();

    }

    /**
     * Add the given component to the Display
     * @param c Compoent to add
     * @return the Component added.
     */
    public Component addToGUI(Component c) {

        content.add(c, JLayeredPane.MODAL_LAYER);

        addedItems.add(c);

        return c;
    }

    /**
     * Removes the given compoent from the Display if
     * it has been added previously.
     * @param c the Component to remove
     */
    public void removeFromGui(Component c) {
        content.remove(c);
    }




}
