package cage.graphics;

import cage.core.Cage;
import cage.core.Logger;
import cage.input.KeyboardHandler;
import cage.input.MouseHandler;
import cage.ui.Overlay;
import cage.core.ResourceEngine;
import cage.graphics.Scene;
import org.minueto.window.MinuetoWindow;

import java.awt.*;
import java.util.LinkedList;
import java.util.Collection;

/**
 * Cage and Carl
 * User: Seth Marinello
 * Date: May 10, 2007
 * Time: 4:43:58 PM.
 */
public class GraphicsEngine {

    /** The log. */
    private static Logger log = Logger.getRootLogger();

    /** The display. */
    private static Display display;

    /** The window. */
    private static MinuetoWindow window;

    /** The anim. */
    static Animation anim;

    /** The scene graphics. */
    private static LinkedList<Sprite> sceneGraphics;

    /** The width to use when initializing this GraphicsEngine's window. */
    private static int width = 800;
    
    /** The height to use when initializing this GraphicsEngine's window. */
    private static int height = 600;

    /** The x scale. */
    private static double xScale = 1.0;
    
    /** The y scale. */
    private static double yScale = 1.0;

    /**
     * Create a Graphics engine and initialize all instance fields.
     * 
     * @param title The window title for the game.
     * @param decorated Indicates if the window should have a titlebar or not.
     */
    public static void setup(String title, boolean decorated) {

        //Initialize the list of sceneGraphics in the current drawing state, organzied by layer
        sceneGraphics = new LinkedList<Sprite>();

        //Make sure the width and height are at least postive
        assert(width > 0 && height > 0);

        if (xScale != 1.0 || yScale != 1.0) {
            scaleGraphics();
        }

        //Initialize the Window to be rendered to by this graphics engine
        display = new Display(width , height , decorated);
        display.setTitle(title);
        window = display.getGraphicsDisplay();

        window.registerMouseHandler(new MouseHandler(), Cage.getQueue());
        window.registerKeyboardHandler(new KeyboardHandler(), Cage.getQueue());

        window.clear(Color.BLACK);

        window.render();

    }


    /**
     * Gets the width.
     * 
     * @return the width
     */
    public static int getWidth() {
        return width;
    }

    /**
     * Sets the width.
     * 
     * @param width the new width
     */
    public static void setWidth(int width) {
        GraphicsEngine.width = width;
    }

    /**
     * Gets the height.
     * 
     * @return the height
     */
    public static int getHeight() {
        return height;
    }

    /**
     * Sets the height.
     * 
     * @param height the new height
     */
    public static void setHeight(int height) {
        GraphicsEngine.height = height;
    }

    /**
     * Gets the x scale.
     * 
     * @return the x scale
     */
    public static double getXScale() {
        return xScale;
    }

    /**
     * Sets the x scale.
     * 
     * @param xScale the new x scale
     */
    public static void setXScale(double xScale) {
        GraphicsEngine.xScale = xScale;
    }

    /**
     * Gets the y scale.
     * 
     * @return the y scale
     */
    public static double getYScale() {
        return yScale;
    }

    /**
     * Sets the y scale.
     * 
     * @param yScale the new y scale
     */
    public static void setYScale(double yScale) {
        GraphicsEngine.yScale = yScale;
    }

    /**
     * Draws the image of a Cage.Graphics.Sprite at its coordinates
     * and adds it to the end of the sceneGraphics vector.
     * public static  void addSprite(Sprite s){
     * sceneGraphics.add(s);
     * }
     * 
     * public static boolean removeSprite(Sprite s) {
     * return sceneGraphics.remove(s);
     * }
     */
    
    public static void render() {
        window.clear(Color.BLACK);

        for(Sprite s : sceneGraphics) {
            if (s.getVisible()) {
                window.draw(s.getImage(), s.getX(), s.getY());
            }
            if (Logger.getRootLogger().getLevel() == Logger.Level.DEBUG) {
                for (HotSpot hs : s.getHotSpots()) {
                    window.draw(hs.getImage(), hs.getX(), hs.getY());
                }
            }
        }

        if (Logger.getRootLogger().getLevel() == Logger.Level.DEBUG) {
        }

        window.render();
    }

    /**
     * Gets the display.
     * 
     * @return the display
     */
    public static Display getDisplay() {
        return display;
    }

    /**
     * Gets the sprite list.
     * 
     * @return the sprite list
     */
    public static LinkedList<Sprite> getSpriteList() {
        return sceneGraphics;
    }

    /**
     * Sets the graphics.
     * 
     * @param list the new graphics
     */
    public static void setGraphics(LinkedList<Sprite> list) {

        sceneGraphics = list;
        render();

    }

    /**
     * Adds the overlay components.
     * 
     * @param overlayContents the overlay contents
     */
    public static void addOverlayComponents(LinkedList<Overlay.NamedPanel> overlayContents) {

        for (Component c : overlayContents) {
            display.addToGUI(c);
        }

    }

    /**
     * Removes the overlay components.
     * 
     * @param overlayContents the overlay contents
     */
    public static void removeOverlayComponents(LinkedList<Overlay.NamedPanel> overlayContents) {

        for (Component c : overlayContents) {
            display.removeFromGui(c);
        }
    }

    /**
     * Removes the all overlay components.
     */
    public static void removeALlOverlayComponents() {
        display.removeGui();
    }

    /**
     * Scale the size of all Visual Items (Animations and Images). Then scale the
     * location of all Sprites (and therefore HotSpots, which are also scaled for size)
     * and then finally the location of overlays are scaled. The actual GUI Components
     * in the Overlays are not resized (just location is changed), they will still fit 
     * all contents they had before.
     */
    public static void scaleGraphics() {
        Collection<VisualItem> images = ResourceEngine.getAllVisualItems();
        Collection<Scene> scenes = ResourceEngine.getAllScenes();
        Collection<Overlay> overlays = ResourceEngine.getAllOverlays();
        Collection<Sprite> sprites;
        Collection<Overlay.NamedPanel> components;


        log.info("Scaling all Images...");
        for (VisualItem i : images) {
            i.scale(xScale, yScale);
        }

        log.info("Scaling all Sprites...");
        for (Scene s : scenes) {
            sprites = s.getGraphics();
            for (Sprite sp : sprites) {
                sp.scale(xScale, yScale);
            }
        }

        log.info("Scaling Overlay Locations...");
        for (Overlay o : overlays) {
            components = o.getComponents();

            for (Overlay.NamedPanel c: components) {
                c.setLocation((int)(c.getX() * xScale), (int)(c.getY() * yScale));
            }
        }


        System.gc();

    }

}
