package cage.graphics;

import org.minueto.image.MinuetoImage;
import org.minueto.image.MinuetoRectangle;
import org.minueto.MinuetoColor;
import cage.core.Logger;

/**
 * A HotSpot is a rectangular region defined by a point and X,Y sizes.
 * This region can have an associated action and should be used to
 * sub-divide a single Sprite into several active regions calling different
 * CARL actions. When debug mode is on, the GraphicsEngine will draw the outlines
 * of the HotSpots using their stored image.
 *
 * @author Seth Marinello
 */
public class HotSpot extends Clickable {

    /** The name of the HotSpot */
    protected String name;

    /** If a HotSpot is active the MouseHandler will call the HotSpot's action when it is clicked */
    protected boolean active;

    /** An image, assigned to a colored rectangle used for debugging only */
    private MinuetoImage image;

    /**
     * Creates a HotSpot with the given parameters.
     * @param name The name of the HotSpot
     * @param actionName The name of the action to call when clicked.
     * @param x The top left X location.
     * @param y The top left Y location.
     * @param sizex The width of the HotSpot.
     * @param sizey The height of the HotSpot.
     * @param active If the HotSpot is active or not.
     */
    public HotSpot(String name, String actionName, int x, int y, int sizex, int sizey, boolean active) {
        topLeft = new Point(x, y);

        width = sizex;
        height = sizey;

        botRight = new Point(sizex + x, sizey + y);

        image = new MinuetoRectangle(sizex, sizey, MinuetoColor.RED, false);

        this.name = name;
        this.active = active;
        this.actionName = actionName;

    }

    /**
     * Gets the rectangle image showing the outine of the HotSpot. For debugging.
     * @return the outline of the HotSpot as an Image.
     */
    public MinuetoImage getImage() {
        return image;
    }

    /**
     * Rescales the HotSpot's location and size by
     * the given X and Y factors.
     * @param xScale The ratio to scale X by.
     * @param yScale The ratio to scale Y by.
     */
    public void scale(double xScale, double yScale) {

        int xSize = botRight.getX() - topLeft.getX();
        int ySize = botRight.getY() - topLeft.getY();

        topLeft.setX((int)(topLeft.getX() * xScale));
        topLeft.setY((int)(topLeft.getY() * yScale));

        botRight.setX((int)(topLeft.getX() + (xSize * xScale)));
        botRight.setY((int)(topLeft.getY() + (ySize * yScale)));

        image = new MinuetoRectangle((int)(xSize * xScale), (int)(ySize * yScale), MinuetoColor.RED, false);
    }

    /**
     * Gets the name of this HotSpot
     * @return The name of the HotSpot
     */
    public String getName() {
        return name;
    }

    /**
     * Creates an XML String representing this HotSpot
     * @return XML representation of this HotSpot object.
     */
    public String toXML() {
        StringBuffer xml = new StringBuffer(1000);

        xml.append("         ");
        xml.append("<hotspot name=\"").append(name).append("\" "
                + "action=\"").append(actionName).append("\" " + "X=\"").
                append(topLeft.getX()).append("\" Y=\"").append(topLeft.getY()).
                append("\" " + "sizeX=\"").append(image.getWidth()).append("\" sizeY=\"").
                append(image.getHeight()).append("\" " + "active=\"").append(active).
                append("\" />");

        return xml.toString();
    }

    /**
     * Sets if this HotSpot should be considered when the mouse is clicked
     * @param active the value to assign to the HotSpot
     */
    public void setActive(boolean active) {
        this.active = active;
    }

    /**
     * Returns true if the HotSpot is active.
     * @return True if the HotSpot is active, false if not.
     */
    public boolean getActive() {
        return active;
    }
}
