package cage.graphics;

import cage.graphics.Sprite;
import cage.input.Binding;
import cage.carl.Closure;
import cage.carl.Carl;
import cage.core.Logger;
import cage.core.ResourceEngine;
import cage.ui.Overlay;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Collection;

/**
 * A Scene is a collection of sprites and possibly an overlay
 * which creates a scene in overall production.
 *
 * @author Seth Marinello
 */
public class Scene {

    /** The name of the Scene. */
    private String name;
    /** The name of the overlay to use */
    private String overlayName;
    /**
     * The name of the action to call when this Scene is made the currentScene. This
     * action is called before the Scene is loaded into the graphics engine.
     */
    private String startActionName;
    /**
     * A list of all the Sprites to display on this scene
     */
    private LinkedList<Sprite> requiredSprites;

    /**
     * A map of keyboard bindings local to this Scene
     */
    private HashMap<Integer, Binding> bindings;

    /**
     * A map of routines local to this Scene
     */
    private HashMap<String, Closure> routines;

    /**
     * Creates a Scene with the specified name.
     * @param name The name of the scene.
     */
    public Scene(String name) {
        this.name = name;
        requiredSprites = new LinkedList<Sprite>();
        bindings = new HashMap<Integer, Binding>();
        routines = new HashMap<String, Closure>();
        ResourceEngine.registerScene(this);
    }

    /**
     * Sets the name of the Overlay resource to be loaded when this scene
     * is displayed.
     * @param overlayName The name of the Overlay resource.
     */
    public void setOverlayName(String overlayName) {
        this.overlayName = overlayName;
    }

    /**
     * Gets the overlay name for the Scene.
     * @return the overlay resource name.
     */
    public String getOverlayName() {
        return overlayName;
    }

    /**
     * Searches the ResourceEngine for the overlay with this
     * Scenes stored overlay name and returns it.
     * @return the result of getOverlay in ResourceEngine.
     */
    public Overlay getOverlay() {
        return ResourceEngine.getOverlay(overlayName);
    }

    /**
     * Gets the name of the Scene
     * @return the name of the Scene
     */
    public String getName() {
        return name;
    }

    /**
     * The name of the action to call when this scene is made the currentScene
     * @return the name of the action to call
     */
    public String getStartActionName() {
        return startActionName;
    }

    /**
     * Sets the action name to call when this scene is made current.
     * @param startActionName the name of the action to call.
     */
    public void setStartActionName(String startActionName) {
        this.startActionName = startActionName;
    }

    /**
     * Returns all of the sprites used in this scene
     * @return LinkedList of all sprites in the scene.
     */
    public LinkedList<Sprite> getGraphics() {
        return requiredSprites;
    }

    /**
     * Add the specified Sprite to the scene.
     * @param s the Sprite to add.
     */
    public void addSprite(Sprite s)  {
        requiredSprites.add(s);
    }

    /**
     * Looks for a sprite with the given name in the scene and
     * returns it if found.
     * @param name The name of the Sprite to search for.
     * @return the Sprite if found.
     */
    public Sprite getSprite(String name) {
        for (Sprite s : requiredSprites) {
            if (name.equals(s.getName())) {
                return s;
            }
        }

        Logger.getRootLogger().error("Could not find sprite " + name);
        
        return null;
    }

    /**
     * Adds a binding to the bindings used for this scene.
     * @param keyConst the value of the key the bind is for
     * @param bind the Binding for the key value. 
     */
    public void addBinding(Integer keyConst, Binding bind) {
        bindings.put(keyConst, bind);
    }

    /**
     * Remove a binding from the bindings used for this scene.
     * @param keyConst the value of the key the bind is for
     */
    public void removeBinding(Integer keyConst) {
        bindings.remove(keyConst);
    }

    /**
     * Returns the map of all bindings on the Scene. Scene bindings
     * have priority over global bindings.
     * @return map of all bindings
     */
    public Map<Integer, Binding> getBindings() {
        return bindings;
    }

    /**
     * Adds an action to be called each engine clock tick for this scene.
     * @param name the name of the action to add
     */
    public void addRoutine(String name) {
        Closure action = Carl.getAction(name);

        if (action != null) {
            routines.put(name, action);
        }
    }

    /**
     * Remove the specifed routine from this scene
     * @param name the name of the routine to remove
     */
    public void removeRoutine(String name) {
        routines.remove(name);
    }

    /**
     * Returns a collection of all Closures to be called each engine tick when this scene
     * is the current scene.
     * @return collection of all Closures
     */
    public Collection<Closure> getRoutines() {
        return routines.values();
    }


    public String toXML() {
        StringBuffer xml = new StringBuffer(1000);

        xml.append("\n   ");
        xml.append("<scene name=\"");
        xml.append(name).append("\"");

        if (requiredSprites.size() == 0 && bindings.size() == 0) {
            xml.append("/>");
        }
        else {
            xml.append(">");

            for (Binding b : bindings.values()) {
                xml.append("\n      ");
                xml.append(b.toXML());
            }

            for (Sprite s : requiredSprites) {
                xml.append("\n");
                xml.append(s.toXML());
            }
            xml.append("\n   ");
            xml.append("</scene>");
        }

        return xml.toString();
    }

    public String toString() {
        return "(Scene) " + name;
    }
}
