package cage.graphics;

import cage.core.Logger;
import cage.core.ResourceEngine;
import org.minueto.image.MinuetoImage;

import java.util.LinkedList;

/**
 * TODO: Sprite needs a way to set the Image!.
 *
 * @author Seth Marinello
 */
public class Sprite extends Clickable {

    /** The log. */
    private static Logger log = Logger.getRootLogger();

    /** The image. */
    protected VisualItem image;

    /** Is the sprite visible. */
    protected boolean visible;

    /** The hot spots. */
    protected LinkedList<HotSpot> hotSpots;

    /** Is the sprite animated. */
    protected boolean animated;

    /** The index. */
    private int index = 0;

    private boolean loop = false;

    private boolean play = false;

    /**
     * Instantiates a new Sprite.
     *
     * @param imageName the image name
     * @param inName the in name
     * @param location the X,Y location of the upper left of the Sprite.
     */
    public Sprite(String imageName, String inName, Point location) {

        image = ResourceEngine.getVisualItem(imageName);
        if (image == null) {
            try {
                log.debug("Loading Sprite " + inName + " Image from file " + imageName
                        + " because there was no registerd Image by the name " + imageName);
                image = new Image(inName, imageName);
            } catch (Exception e) {
                log.error("Could not load Image " + imageName + " for Sprite " +  inName, e);
            }
        }

        if (inName == null) {
            name = image.getName();
        }
        else {
            name = inName;
        }

        //If the image is a animation, behave as a animation sprite
        animated = image instanceof Animation;


        topLeft = location;

        width = image.getWidth();
        height = image.getHeight();

        botRight = new Point(topLeft.getX() + width, topLeft.getY() + height);

        visible = true;

        hotSpots = new LinkedList<HotSpot>();

    }


    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        try {
            Sprite invader = (Sprite) obj;

            if(this.name.equals(invader.getName()) && this.image.equals(invader.image)) {
                return true;
            }
        } catch (Exception e) {
            log.error("Sprite.equals() became confused", e);
        }

        return false;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return  "(Sprite) " + name + " @ location: " + topLeft;
    }

    /**
     * Checks if is animated.
     *
     * @return true, if is animated
     */
    public boolean getAnimated() {
        return animated;
    }

    /**
     * Gets loop.
     *
     * @return loop
     */
    public boolean getLoop() {
        return loop;
    }

    /**
     * Sets loop.
     *
     * @param loop new loop value
     */
    public void setLoop(boolean loop) {
        this.loop = loop;
    }

    /**
     * Gets play.
     *
     * @return play
     */
    public boolean getPlay() {
        return play;
    }

    /**
     * Sets play.
     *
     * @param play new play
     */
    public void setPlay(boolean play) {
        this.play = play;
    }

    /**
     * Gets the frame in the animation.
     *
     * @return the frame
     */
    public MinuetoImage getFrame() {
        MinuetoImage ret = null;
        Animation anim = (Animation)image;
        int size = anim.getFrameCount();

        if (play) {
            if (index >= size) {
                if (loop) {
                    index = 0;
                }
                else {
                    play = false;
                    index = size - 1;
                }
                ret = anim.getFrame(index);
            }
            else {
                ret = anim.getFrame(index);
                index++;
            }
        }
        else {
            ret = anim.getFrame(index);
        }

        return ret;
    }

    /**
     * Gets the visual item.
     *
     * @return the visual item
     */
    public VisualItem getVisualItem() {
        return image;
    }

    /**
     * Gets the image.
     *
     * @return the image
     */
    public MinuetoImage getImage() {
        if (!animated) {
            return image.getImage();
        }
        else {
            return getFrame();
        }
    }

    /**
     * Set the location of this sprite to the specified point.
     * Also shifts all of the hot spots on the sprite to match
     * the new location.
     *
     * @param coord the X,Y location of the sprite
     */
    public void setLocation(Point coord) {
        setLocation(coord.getX(), coord.getY());
    }

    /**
     * Set the location of this sprite to the specified point.
     * Also shifts all of the hot spots on the sprite to match
     * the new location.
     *
     * @param x the x coord
     * @param y the y coord
     */
    public void setLocation(int x, int y) {
        super.setLocation(x, y);

        int xShift = x - topLeft.getX();
        int yShift = y - topLeft.getY();

        for (HotSpot s : hotSpots) {
            s.setLocation(s.getX() + xShift, s.getY()+ yShift);
        }
    }

    /**
     * Gets the action name.
     *
     * @return the action name
     */
    public String getActionName() {
        return actionName;
    }

    /**
     * Sets the action name.
     *
     * @param actionName the new action name
     */
    public void setActionName(String actionName) {
        this.actionName = actionName;
    }

    /**
     * Scale the Sprite location and image by the given
     * X and Y scales, as ratios.
     *
     * @param xScale the x scale
     * @param yScale the y scale
     */
    public void scale(double xScale, double yScale) {
        for (HotSpot s : hotSpots) {
            s.scale(xScale, yScale);
        }

        topLeft.setX((int)(topLeft.getX() * xScale));
        topLeft.setY((int)(topLeft.getY() * yScale));

        this.botRight = new Point(topLeft.getX() + width,
                topLeft.getY() + height);

        //setLocation(new Point((int)(getX() * xScale), (int)(getY() * yScale)));
    }

    /**
     * Sets if the Sprite is visible.
     *
     * @param b the new visible value
     */
    public void setVisible(boolean b) {
        visible = b;
    }

    /**
     * Checks if the Sprite is visible.
     *
     * @return true, if is visible
     */
    public boolean getVisible() {
        return visible;
    }

    /**
     * Gets the hot spots.
     *
     * @return the hot spots
     */
    public LinkedList<HotSpot> getHotSpots() {
        return hotSpots;
    }

    /**
     * Adds the hot spot.
     *
     * @param s the HotSpot to add.
     */
    public void addHotSpot(HotSpot s) {
        hotSpots.add(s);
    }

    /**
     * Creates an XML String representing this Sprite
     * @return XML representation of this Sprite object.
     */
    public String toXML() {
        StringBuffer xml = new StringBuffer(1000);

        String actionString = actionName == null ? "" : "action=\"" + actionName + "\" ";

        xml.append("      ");
        xml.append("<sprite name=\"" + name + "\" "
                + "src=\"" + image.getName() + "\" " + actionString
                + "X=\"" + topLeft.getX() + "\" Y=\"" + topLeft.getY() + "\" "
                + "loop=\"" + loop + "\" play=\"" + play + "\" "
                + "visible=\"" + visible + "\"");

        if (hotSpots.size() == 0) {
            xml.append("/>");
        }
        else {
            xml.append(">");
            for (HotSpot s : hotSpots) {
                xml.append("\n" + s.toXML());
            }
            xml.append("\n      ");
            xml.append("</sprite>");
        }

        return xml.toString();
    }

    /**
     * Gets the hot spot.
     *
     * @param name the name
     *
     * @return the hot spot
     */
    public HotSpot getHotSpot(String name) {
        for (HotSpot s : hotSpots) {
            if (name.equals(s.getName())) {
                return s;
            }
        }

        Logger.getRootLogger().error("Could not find HotSpot " + name);

        return null;
    }
}