package cage.input;

import org.minueto.handlers.MinuetoMouseHandler;
import org.minueto.handlers.MinuetoMouse;
import cage.core.Logger;
import cage.core.Cage;
import cage.graphics.Scene;
import cage.graphics.*;

import java.util.LinkedList;

/**
 * Handles mouse clicks and checks clickable items in the current scene to see if something was hit.
 * @author Seth Marinello
 */
public class MouseHandler implements MinuetoMouseHandler {

    /** The log. */
    private Logger log = Logger.getRootLogger();

    /** The selected. */
    private static Sprite selected;

    /** Edit mode allows the movement of spites with right click. */
    public static boolean editMode = false;


    /**
     * If the right button is clicked, the closest sprite is selected. (Only in edit mode)
     * @param iX X position of press
     * @param iY Y position of the press
     * @param iButton The button pressed
     */
    public void handleMousePress(int iX, int iY, int iButton) {

        log.debug("Mouse click on button " + iButton + " detected at " + iX + "," + iY);

        if (iButton == MinuetoMouse.MOUSE_BUTTON_RIGHT && editMode) {
            Scene currentScene = Cage.getCurrentScene();

            LinkedList<Sprite> sprites = currentScene.getGraphics();

            for (Sprite s : sprites) {
                if (s.inputCollision(iX, iY ) && s.getVisible()) {
                    selected = s;
                }
            }
        }
    }

    /**
     * Called when the mouse is released. If the left button is released, the location is compared to
     * Sprites and HotSpots on the current Scene and the action is called for the furthest forward
     * item hit by the click.
     *
     * If the right button is released, whatever was selected is deselected. Only useful
     * in edit mode.
     * @param iX X position of release.
     * @param iY Y position of the release
     * @param iButton The button released
     */
    public void handleMouseRelease(int iX, int iY, int iButton) {

        log.debug("Mouse release on button " + iButton + " detected at " + iX + "," + iY);

        if (iButton == MinuetoMouse.MOUSE_BUTTON_LEFT) {
            Scene currentScene = Cage.getCurrentScene();

            LinkedList<Sprite> sprites = currentScene.getGraphics();
            LinkedList<HotSpot> spots;

            Clickable target = null;

            for (Sprite s : sprites) {
                if (s.inputCollision(iX, iY ) && s.getVisible()) {
                    spots = s.getHotSpots();

                    target = s;

                    for (HotSpot spot : spots) {
                        log.debug("Checking HotSpot " + spot.getName()
                                + " at (" + spot.getX() + ", " + spot.getY() + ") and ("
                                + spot.getImage().getWidth() + ", " + spot.getImage().getHeight() + ")"
                                + " and is " + (spot.getActive() ? "active" : "inactive"));
                        if (spot.inputCollision(iX, iY) && spot.getActive()) {
                            log.debug("Selected " + spot.getName() + " as target");
                            target = spot;
                        }
                    }
                }
            }

            if (target != null) {
                log.debug("Clicked on " + target.getName() + " at (" + iX + "," + iY + ")");
                target.performAction();
            }
        }
        else if (iButton == MinuetoMouse.MOUSE_BUTTON_RIGHT) {
            selected = null;
        }

        if (selected != null && editMode) {
            log.debug("Release at: " + selected.getX() + ", " + selected.getY());
        }

    }


    /**
     * If an item is selected (in edit mode) the location of that
     * selected item is updated to the location of the mouse pointer.
     * @param iX X location
     * @param iY Y location
     */
    public void handleMouseMove(int iX, int iY) {
        if (selected != null && editMode) {
            selected.setLocation(iX, iY);
        }
    }
}
