/*
 * @(#)MinuetoColor.java
 *
 * Minueto - The Game Development Framework 
 * Copyright (c) 2004 McGill University
 * 3480 University Street, Montreal, Quebec H3A 2A7
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.minueto;

import java.awt.Color;
import java.io.Serializable;

/**
 * The <code>MinuetoColor</code> class is used to store information describing a 
 * color. Several prefined colors defined as constants can also be found in this
 * class.
 * <p>
 * Note: Minueto defines a color as a RGB value (red/green/blue). If you are new
 * to computer graphics, please note that this color scheme is significantly different
 * from the one used for paper (cyan, blue, yellow, black, also known as CMYK). 
 *
 * @author	Alexandre Denault
 * @version 1.1
 * @since 	Minueto 0.4
 **/
 public class MinuetoColor implements Serializable {	
	
	private static final long serialVersionUID = 6077031996981772626L;

	private final static String ID = MinuetoColor.class.getName();

	private int iRed;
	private int iBlue;
	private int iGreen;
	private int iAlpha;
	
	/** The color Red. */
	public static final MinuetoColor RED 	= new MinuetoColor( 255, 0, 0);
	/** The color Green. */
	public static final MinuetoColor GREEN 	= new MinuetoColor( 0, 255, 0);
	/** The color Blue. */
	public static final MinuetoColor BLUE 	= new MinuetoColor( 0, 0, 255);
	/** The color Yellow.*/
	public static final MinuetoColor YELLOW = new MinuetoColor( 255, 255, 0);
	/** The color Black.*/
	public static final MinuetoColor BLACK 	= new MinuetoColor( 0, 0, 0);
	/** The color White */
	public static final MinuetoColor WHITE 	= new MinuetoColor( 255, 255, 255);
	
	
	/**
	 * Creates a new RGB color. Colors are defined as three integer
	 * values ranging from 0 to 255. 
	 *
	 * @param iRed <code>int</code> denoting the red color value.
	 * @param iGreen <code>int</code> denoting the green color value.
	 * @param iBlue <code>int</code> denoting the blue color value.
	 *
	 * @throws <code>MinuetoInvalidColorValueException</code> if an invalid color value is passed.
	 **/
	public MinuetoColor(int iRed, int iGreen, int iBlue) {

		if ( (iRed < 0) || (iRed > 255) ) 
			throw new MinuetoInvalidColorValueException("The red color value cannot be " +iRed);
		if ( (iGreen < 0) || (iGreen > 255) ) 
			throw new MinuetoInvalidColorValueException("The green color value cannot be " +iGreen);
		if ( (iBlue < 0) || (iBlue > 255) ) 
			throw new MinuetoInvalidColorValueException("The blue color value cannot be " +iBlue);

		this.iRed = iRed;
		this.iGreen = iGreen;
		this.iBlue = iBlue;	
		this.iAlpha = 255;
   	}
	
	/**
	 * Creates a new RGBA color. Colors are defined as four integers
	 * values ranging from 0 to 255. 
	 *
	 * @param iRed <code>int</code> denoting the red color value.
	 * @param iGreen <code>int</code> denoting the green color value.
	 * @param iBlue <code>int</code> denoting the blue color value.
	 * @param iAlpha <code>int</code> denothing the alpha color value.
	 *
	 * @throws <code>MinuetoInvalidColorValueException</code> if an invalid color value is passed.
	 **/
	public MinuetoColor(int iRed, int iGreen, int iBlue, int iAlpha) {

		if ( (iRed < 0) || (iRed > 255) ) 
			throw new MinuetoInvalidColorValueException("The red color value cannot be " +iRed);
		if ( (iGreen < 0) || (iGreen > 255) ) 
			throw new MinuetoInvalidColorValueException("The green color value cannot be " +iGreen);
		if ( (iBlue < 0) || (iBlue > 255) ) 
			throw new MinuetoInvalidColorValueException("The blue color value cannot be " +iBlue);
		if ( (iAlpha < 0) || (iAlpha > 255) ) 
			throw new MinuetoInvalidColorValueException("The alpha color value cannot be " + iAlpha);

		this.iRed = iRed;
		this.iGreen = iGreen;
		this.iBlue = iBlue;				
		this.iAlpha = iAlpha;
   	}
   	
	
	/**
	 * Creates a new RGB color. Colors are defined as three floating-point
	 * values ranging from 0.0 to 1.0. 
	 *
	 * @param dRed <code>double</code> denoting the red color value.
	 * @param dGreen <code>double</code> denoting the green color value.
	 * @param dBlue Double <code>double</code> the blue color value.
	 *
	 * @throws <code>MinuetoInvalidColorValueException</code> if an invalid color value is passed.
	 **/
   	public  MinuetoColor(double dRed, double dGreen, double dBlue) {
	
		if ( (dRed < 0.0) || (dRed > 1.0) ) 
			throw new MinuetoInvalidColorValueException("The red color value cannot be " +dRed);
		if ( (dGreen < 0.0) || (dGreen > 1.0) ) 
			throw new MinuetoInvalidColorValueException("The green color value cannot be " +dGreen);
		if ( (dBlue < 0.0) || (dBlue > 1.0) ) 
			throw new MinuetoInvalidColorValueException("The blue color value cannot be " +dBlue);
			
		this.iRed = Math.round((int)(255.0*dRed));
		this.iGreen = Math.round((int)(255.0*dGreen));
		this.iBlue = Math.round((int)(255.0*dBlue));
		this.iAlpha = 255;
   	}
   	
	/**
	 * Creates a new RGBA color. Colors are defined as four floating-point
	 * values ranging from 0.0 to 1.0. 
	 *
	 * @param dRed <code>double</code> denoting the red color value.
	 * @param dGreen <code>double</code> denoting the green color value.
	 * @param dBlue <code>double</code> denoting the blue color value.
	 * @param dAlpha <code>double</code> denoting the alpha color value. 
	 *
	 * @throws <code>MinuetoInvalidColorValueException</code> if an invalid color value is passed.
	 **/
   	public  MinuetoColor(double dRed, double dGreen, double dBlue, double dAlpha) {
	
		if ( (dRed < 0.0) || (dRed > 1.0) ) 
			throw new MinuetoInvalidColorValueException("The red color value cannot be " +dRed);
		if ( (dGreen < 0.0) || (dGreen > 1.0) ) 
			throw new MinuetoInvalidColorValueException("The green color value cannot be " +dGreen);
		if ( (dBlue < 0.0) || (dBlue > 1.0) ) 
			throw new MinuetoInvalidColorValueException("The blue color value cannot be " +dBlue);
		if ( (dAlpha < 0.0) || (dAlpha > 1.0) ) 
			throw new MinuetoInvalidColorValueException("The alpha color value cannot be " +dAlpha);
			
		this.iRed = Math.round((int)(255.0*dRed));
		this.iGreen = Math.round((int)(255.0*dGreen));
		this.iBlue = Math.round((int)(255.0*dBlue));
		this.iAlpha = Math.round((int)(255.0*dAlpha));
   	}

   	/**
   	 * Returns a darker color. The darkness factor is determined by the double value, whose value should 
   	 * be between 0.0 and 1.0.
   	 *  
   	 * @param factor <code>double</code> that determines how darker the color will be.
   	 * @return <code>MinuetoColor</code> darker color.
   	 */
   	public MinuetoColor darken(double factor) {
		
		return this.darken((int)(255.0*factor));
	}
   	
   	/**
   	 * Returns a darker color. The darkness factor is determined by the integer value, whose value should 
   	 * be between 0 and 255.
   	 *  
   	 * @param factor <code>double</code> that determines how darker the color will be.
   	 * @return <code>MinuetoColor</code> darker color.
   	 */
   	public MinuetoColor darken(int factor) {
		
   		if (factor > 255) factor = 255;
   		if (factor < 0) factor = 0;
   		
   		int iNewRed = this.iRed - factor;		
		int iNewGreen = this.iGreen - factor;
		int iNewBlue = this.iBlue - factor;
		
		return new MinuetoColor(
				(iNewRed > 0) ? iNewRed : 0,
				(iNewGreen > 0) ? iNewGreen : 0,
				(iNewBlue > 0) ? iNewBlue : 0,
				iAlpha);						
	}
	
   	/**
   	 * Returns a lighter color. The lightness factor is determined by the double value, whose value should 
   	 * be between 0.0 and 1.0.
   	 *  
   	 * @param factor <code>double</code> that determines how lighter the color will be.
   	 * @return <code>MinuetoColor</code> lighter color.
   	 */
   	public MinuetoColor lighten(double factor) {
		
		return this.lighten((int)(255.0*factor));
	}
	
   	/**
   	 * Returns a lighter color. The lightness factor is determined by the double value, whose value should 
   	 * be between 0.0 and 1.0.
   	 *  
   	 * @param factor <code>double</code> that determines how lighter the color will be.
   	 * @return <code>MinuetoColor</code> lighter color.
   	 */
   	public MinuetoColor lighten(int factor) {
		
   		if (factor > 255) factor = 255;
   		if (factor < 0) factor = 0;
   		
   		int iNewRed = this.iRed + factor;		
		int iNewGreen = this.iGreen + factor;
		int iNewBlue = this.iBlue + factor;
		
		return new MinuetoColor(
				(iNewRed < 255) ? iNewRed : 255,
				(iNewGreen < 255) ? iNewGreen : 255,
				(iNewBlue < 255) ? iNewBlue : 255,
				iAlpha);
	}
   	
   	/**
   	 * Returns the blue value of this color. The returned value is an integer 
   	 * whose value is between 0 and 255.
   	 * 
   	 * @return <code>int</code> denoting the blue value of this color.
   	 */
   	public int getBlue() {
   		
   		return this.iBlue;
   	}
   	
   	/**
   	 * Returns the red value of this color. The returned value is an integer 
   	 * whose value is between 0 and 255.
   	 * 
   	 * @return <code>int</code> denoting the red value of this color.
   	 */
   	public int getRed() {
   		
   		return this.iRed;
   	}
   	
   	/**
   	 * Returns the green value of this color. The returned value is an integer 
   	 * whose value is between 0 and 255.
   	 * 
   	 * @return <code>int</code> denoting the green value of this color.
   	 */
   	public int getGreen() {
   		
   		return this.iGreen;
   	}
   	
   	/**
   	 * Returns the alpha value of this color. The returned value is an integer 
   	 * whose value is between 0 and 255.
   	 * 
   	 * @return <code>int</code> denoting the alpha value of this color.
   	 */
   	public int getAlpha() {
   		
   		return this.iAlpha;
   	}
   	
   	/**
   	 * Returns the equivalent AWT color object.
   	 *  
   	 * @return <code>Color</code> Color code of the object.
   	 */
   	public Color getAWTColor() {
   		
   		return new Color(this.iRed, this.iGreen, this.iBlue, this.iAlpha);
   	}
   	
   	/**
   	 * Returns the ARGB color code for this color as an integer.
   	 * 
   	 * @return <code>int</code> of the ARGB color code.
   	 */
   	public int getARGBColorValue() {
   		
   		return new Color(this.iRed, this.iGreen, this.iBlue, this.iAlpha).getRGB();
   	}
   	
   	/**
   	 * Returns a string representation of the color.
   	 * 
   	 * @return <code>String</code> containing the color information.
   	 */
   	public String toString() {
   		return ID + " <Red: " + this.iRed + " Green: " + this.iGreen + " Blue: " + this.iBlue + " Alpha: " + this.iAlpha + " >";  
   	}
}
