/*
 * @(#)MinuetoImage.java
 *
 * Minueto - The Game Development Framework 
 * Copyright (c) 2004 McGill University
 * 3480 University Street, Montreal, Quebec H3A 2A7
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.minueto.image;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import org.minueto.MinuetoColor;
import org.minueto.MinuetoFileException;
import org.minueto.MinuetoOutOfBoundException;
import org.minueto.MinuetoZeroNegativeException;

/**
 * The <code>MinuetoImage</code> class describes an image that can be drawn on a 
 * <code>MinuetoWindow</code>. All other types of images used by Minueto are subclasses
 * of <code>MinuetoImage</code>.
 * <p>
 * Basic 2D image transformations (such as crop, scale, rotate) can be executed
 * on any <code>MinuetoImage</code>. These transformation produce new <code>MinuetoImage</code> that can
 * be manipulated independently from their original image.Please note that these
 * transformations are slow and should not be used in realtime. Results from 
 * these operations should be preferably cached in advanced.
 * <p>
 * Images are created at the same colordepth as <code>MinuetoWindow</code>. When created, a
 * <code>MinuetoImage</code> is transparent. Other <code>MinuetoImage</code> can be draw over it, thus
 * allowing you to build more complexe composite images.
 * <p>
 *
 * @author	Alexandre Denault
 * @version 1.0
 * @since 	Minueto 0.4
 **/

public class MinuetoImage implements Cloneable {

	private BufferedImage	bufImage;
	private Graphics2D 		g2dGraphics;
	private int				iWidth;
	private int				iHeight;
	private boolean			bAccelerated;

	/**
	 * Create a blank (transparent) image of the specified width and height.
	 *
	 * @param iX <code>int</code> int denoting the width of the blank image.
	 * @param iY <code>int</code< int denoting the height of the blank image.
	 * @throws <code>MinuetoZeroNegativeException</code> if the image size is invalid.
	 **/
	public MinuetoImage(int iX, int iY) {

		if (iX < 1) throw new MinuetoZeroNegativeException("Cannot create image of width " + iWidth);
		if (iY < 1) throw new MinuetoZeroNegativeException("Cannot create image of height " + iHeight);

		this.setUpImage(ImageTools.createImage(iX, iY));
	}

	/**
	 * Create an image using an existing buffered image.
	 *
	 * @param image <code>BufferedImage</code> witht he content of the image. 
	 **/
	public MinuetoImage(BufferedImage image) {
		
		this.setUpImage(image);
	}
	
	/**
	 * Protected constructor used by the package to create a <code>MinuetoImage</code>
	 * without a buffered image. The buffered image will be latter added
	 * by the <code>setUpImage</code> method.
	 **/
	protected MinuetoImage() {

	}

	/**
	 * Protected method used to set up a <code>MinuetoImage</code>. All <code>MinuetoImage</code> will,
	 * at one point in their lifetime, go throught this method.
	 *
	 * @param	bufImage <code>BufferedImage</code> storing the content of the
	 *				image.
	 **/
	protected void setUpImage(BufferedImage bufImage) {

		if (bufImage == null) new NullPointerException();

		this.bufImage = bufImage;

		this.iWidth 		= this.bufImage.getWidth();
		this.iHeight 		= this.bufImage.getHeight();
		this.g2dGraphics 	= this.bufImage.createGraphics();
		
		this.accelerateBuffer();
	}

	/**
	 * Return a cropped copy of this <code>MinuetoImage</code>. The action of cropping an
	 * image can be described as copying a portion of an image to a new image.
	 *
	 * @param iCornerX <code>int</code> denoting the X coordinate of the upper left corner
	 *                 of the old image to be used as the upper left corner of the new image..
	 * @param iCornerY <code>int</code> denoting the Y coordinate of the upper left corner.
	 *                 of the old image to be used as the upper left corner of the new image.
	 * @param iSizeX <code>int</code> denoting the width of the portion of the old image to
	 *                 copy in the new image.
	 * @param iSizeY <code>int</code> denoting the height of the portion of the old image to
	 *                 copy in the new image.
	 * @return <code>MinuetoImage</code> of the cropped image.
	 * @throws <code>MinuetoZeroNegativeException</code> if an invalid position or size is used.
	 * @throws <code>MinuetoOutOfBoundException</code> if a
	 **/
	public MinuetoImage crop(int iCornerX, int iCornerY, int iSizeX, int iSizeY) {

		/* Resulting MinuetoImage */
		MinuetoImage	mimDestinationImage;
		
		if (iCornerX < 0) throw new MinuetoZeroNegativeException("Cannot start crop at X position " + iCornerX);
		if (iCornerY < 0) throw new MinuetoZeroNegativeException("Cannot start crop at Y position " + iCornerY);
		if (iSizeX < 0) throw new MinuetoZeroNegativeException("Cannot end crop at X position " + iSizeX);
		if (iSizeY < 0) throw new MinuetoZeroNegativeException("Cannot end crop at Y position " + iSizeY);

		if (iCornerX >= this.iWidth) throw new MinuetoOutOfBoundException("Cannot start crop at X position " + iCornerX);
		if (iCornerY >= this.iHeight) throw new MinuetoOutOfBoundException("Cannot start crop at Y position " + iCornerY);
		if ((iCornerX+iSizeX) > this.iWidth) throw new MinuetoOutOfBoundException("Cannot end crop at X position " + (iCornerX+iSizeX));
		if ((iCornerY+iSizeY) > this.iHeight) throw new MinuetoOutOfBoundException("Cannot end crop at Y position " + (iCornerY+iSizeY));

		mimDestinationImage = new MinuetoImage(iSizeX, iSizeY);

		mimDestinationImage.g2dGraphics.drawImage(this.getImage().getSubimage(iCornerX, iCornerY, iSizeX, iSizeY),0,0,null);

		return mimDestinationImage;
	}

	/**
	 * Return a rotated copy of this <code>MinuetoImage</code>. The angle must be supplied 
	 * in radian.
	 *
	 * @param dAngle <code>double</code> denoting the rotation angle in radian.
	 * @return <code>MinuetoImage</code> of the rotated image.
	 **/
	public MinuetoImage rotate(double dAngle) {

		/* Reference to the destination image. */
		BufferedImage bufDestinationImage;
		/* Reference to the affine transformation (the rotation) */
		AffineTransform aftTransform;
		/* Reference to the GraphicPointer */
		Graphics2D g2dGraphicsDestination;
		/* Resulting MinuetoImage */
		MinuetoImage	mimDestinationImage;
		/* Size of the resulting MinuetoImage */
		int	iNewHeight;
		int iNewWidth;
		
		
		double x1,x2,x3,y1,y2,y3,xMin,xMax,yMin,yMax, cosAngle, sinAngle;
		
		/* Set up the affine transformation needed to rotate the image */
		aftTransform = new AffineTransform();
		
		/* Rotate the image by dAngle, centered on iWidth/2 and iHeight/2 */
		aftTransform.rotate(dAngle, (double)this.iWidth/2.0, (double)this.iHeight/2.0);		

		cosAngle = Math.cos(dAngle);
		sinAngle = Math.sin(dAngle);

		/* Get the size of our new image */
		/**
		* Before simplification
		* x0 = 0 * cosAngle - 0 * sinAngle;
		* x1 = 0 * cosAngle - this.iHeight * sinAngle;
		* x2 = this.iWidth * cosAngle - 0 * sinAngle;
		* x3 = this.iWidth * cosAngle - this.iHeight * sinAngle;
		*
		* y0 = 0 * sinAngle + 0 * cosAngle;
		* y1 = 0 * sinAngle + this.iHeight * cosAngle;
		* y2 = this.iWidth * sinAngle + 0 * cosAngle;
		* y3 = this.iWidth * sinAngle + this.iHeight * cosAngle;
		*
		* xMin = Math.min( Math.min(x0, x1), Math.min(x2, x3) );
		* xMax = Math.max( Math.max(x0, x1), Math.max(x2, x3) );
		*
		* yMin = Math.min( Math.min(y0, y1), Math.min(y2, y3) );
		* yMax = Math.max( Math.max(y0, y1), Math.max(y2, y3) ); 
		**/
		
		x1 = - this.iHeight * sinAngle;
		x2 = this.iWidth * cosAngle;
		x3 = x2 + x1;
		
		y1 = this.iHeight * cosAngle;
		y2 = this.iWidth * sinAngle;
		y3 = y2 + y1;
		
		xMin = Math.min( Math.min(0, x1), Math.min(x2, x3) );
		xMax = Math.max( Math.max(0, x1), Math.max(x2, x3) );
		
		yMin = Math.min( Math.min(0, y1), Math.min(y2, y3) );
		yMax = Math.max( Math.max(0, y1), Math.max(y2, y3) );
		
		iNewWidth = (int)Math.ceil(Math.abs(xMax - xMin));
		iNewHeight = (int)Math.ceil(Math.abs(yMax - yMin));
		
		iNewWidth = (int)Math.ceil(Math.abs(xMax - xMin));
		iNewHeight = (int)Math.ceil(Math.abs(yMax - yMin));
		
		//System.out.println("Debug: " + iNewWidth + " , " + iNewHeight + " , " +  dAngle);
		
		/* The rotation normaly produces an unaccelerated buffered image.
		 * Instead, we produce an accelerated buffered for the result.
		 * The maximum size this image can be is 141% larger than the original.*/
		bufDestinationImage = ImageTools.createImage(iNewWidth, iNewHeight);
		/* Reference to the GraphicPointer of the destination image.*/
		g2dGraphicsDestination = bufDestinationImage.createGraphics();
		
		/* Make sure we will draw in the middle of our new image. */
		g2dGraphicsDestination.translate( (this.iWidth-iNewWidth)/-2, (this.iHeight-iNewHeight)/-2 );
		
		/* Draw the rotated image. */
		g2dGraphicsDestination.drawImage(	this.bufImage,aftTransform,
											null);

		/* Set up the new MinuetoImage with the rotated image. */
		mimDestinationImage = new MinuetoImage();
		mimDestinationImage.setUpImage(bufDestinationImage);

		return mimDestinationImage;
	}

	/**
	 * Return a scaled copy of this <code>MinuetoImage</code>. The parameters are described as 
	 * scale factors. Thus, calling the scale function with the 2,2 parameters 
	 * would create a image of double width and height.
	 *
	 * @author Michael A. Hawker
	 * @param dFactorX <code>double</code> denoting how the image should be scaled on the X axis.
	 * @param dFactorY <code>double</code> denoting how the image should be scaled on the Y axis.
	 * @return <code>MinuetoImage</code> of the scaled image.
	 * @throws <code>MinuetoZeroNegativeException</code> if a wrong scale factor is used.
	 **/	
	public MinuetoImage scale(double dFactorX, double dFactorY) {

		if (dFactorX > 0 && dFactorY > 0) {
			return this.scaleInternal(dFactorX, dFactorY);
		} else {
			throw new MinuetoZeroNegativeException("Cannot scale an image by a factor of " +dFactorX + " by " + dFactorY);
		}
	}
	
	/**
	 * The mask method creates a new <code>MinuetoImage</code> where
	 * all the non-transparent pixels of this image are replaced 
	 * with the provided color.
	 * 
	 * @param color <code>MinuetoColor</code> denoting the color of the mask.
	 * @return <code>MinuetoImage</code> of the mask
	 */
	public MinuetoImage mask(MinuetoColor color) {
		
		BufferedImage resultImage; // To store the result of the mask
		MinuetoImage result = new MinuetoImage(); // Image we will return
		ImageTransformations it = new ImageTransformations(); // Transformation
		// for the mask
		
		// Apply the mask transformation
		resultImage = it.applyFilter(bufImage, new MaskImageFilter(color));
		
		// Set up the resulting image
		result.setUpImage(resultImage);
		 
		return result;		
				
	}

	/**
	 * Return a scaled copy of this <code>MinuetoImage</code>. The parameters are described as 
	 * scale factors. Thus, calling the scale function with the 2,2 parameters 
	 * would create a image of double width and height.
	 *
	 * @author Michael A. Hawker
	 * @since 0.45
	 * @param dFactorX <code>double</code> denoting how the image should be scaled on the X axis.
	 * @param dFactorY <code>double</code> denoting how the image should be scaled on the Y axis.
	 * @return <code>MinuetoImage</code> of the scaled image.
	 * REMOVED @throws <code>MinuetoZeroNegativeException</code> if a wrong scale factor is used.
	 **/	
	private MinuetoImage scaleInternal(double dFactorX, double dFactorY) {

		/* Reference to the destination image. */
		BufferedImage bufDestinationImage;
		/* Reference to the affine transformation (the scale) */
		AffineTransform aftTransform;
		/* Reference to the GraphicPointer */
		Graphics2D g2dGraphicsDestination;
		/* Resulting MinuetoImage */
		MinuetoImage	mimDestinationImage;

		/* Set up the affine transformation needed to scale the image */
		aftTransform = new AffineTransform();
		/* Scale the image by dFacorX and dFactorY */
		aftTransform.scale(dFactorX, dFactorY);

		/* ADDED: Translate to Account for Negative Scaling Factors */
		if (dFactorX < 0 && dFactorY < 0) {
			aftTransform.translate(this.iWidth * dFactorX, this.iHeight * dFactorY);
		}
		else if(dFactorX < 0) {
			aftTransform.translate(this.iWidth * dFactorX, 0);
		}
		else if(dFactorY < 0) {
			aftTransform.translate(0, this.iHeight * dFactorY);
		}

		/* The translation normaly produces an unaccelerated buffered image.
		 * Instead, we produce an accelerated buffered for the result. */   	  
		/* Math.abs to allow for negative scaling factors */
		bufDestinationImage = ImageTools.createImage((int) Math.abs(dFactorX*this.iWidth), (int) Math.abs(dFactorY*this.iHeight));
		/* Reference to the GraphicPointer of the destination image.*/
		g2dGraphicsDestination = bufDestinationImage.createGraphics();
		/* Draw the scaled image. */
		g2dGraphicsDestination.drawImage(this.bufImage,aftTransform ,null);

		/* Set up the new MinuetoImage with the scaled image. */
		mimDestinationImage = new MinuetoImage();
		mimDestinationImage.setUpImage(bufDestinationImage);

		return mimDestinationImage;
	}

	/**
	 * Returns a flipped copy of this <code>MinuetoImage</code>. The image can be
	 * flipped vertically, horizontally, or both. For example, <code>flip(true, 
	 * false)</code> would flip the image horizontally.
	 *
	 * @author Michael A. Hawker
	 * @since 0.45
	 * @param bHorizontal <code>boolean</code> if the image should be flipped on the X axis.
	 * @param bVertical <code>boolean</code> if the image should be flipped on the Y axis.
	 * @return <code>MinuetoImage</code> of the flipped image.
	 **/	
	public MinuetoImage flip(boolean bHorizontal, boolean bVertical) {
		/* Resulting Minueto Image */
		MinuetoImage mimDestinationImage;

		if (bHorizontal == false && bVertical == false) {
			// Have Nothing To Do But Return Original Image
			mimDestinationImage = new MinuetoImage(this.iWidth, this.iHeight);
			mimDestinationImage.g2dGraphics.drawImage(this.getImage(), 0, 0,null);

			return mimDestinationImage;
		}
		else {
			/* What we want to flip */
			int iFlipX = 1, iFlipY = 1;

			/* Test our Arguments and Convert to Useable Values */
			if (bHorizontal == true) {
				iFlipX = -1;
			}

			if (bVertical == true) {
				iFlipY = -1;
			}

			// Return a Negatively Scaled Image
			return this.scaleInternal(iFlipX, iFlipY);
		}
	}

	/**
	 * Change the color of pixel at location X,y.
	 * <p>
	 * Note: Pixel manipulation of a <code>MinuetoImage</code> is a very slow operation. It should 
	 * not be used in realtime. Consider instead creating small <code>MinuetoImage</code> and drawing
	 * them on this <code>MinuetoImage</code> instead.
	 *
	 * @param mcoColor <code>MinuetoColor</code> denoting the new color for pixel iX,iY.
	 * @param iX <code>int</code> denoting which pixel on the X axis should be colored.
	 * @param iY <code>int</code> denoting which pixel on the y axis should be colored.
	 */
	/*public void setPixel(MinuetoColor mcoColor, int iX, int iY)  {

		// You should really implement this
	}*/

	/**
	* Get the color of the pixel at location x,y. 
	* <p>
	* Note: Pixel manipulation of a <code>MinuetoImage</code> is a very slow operation. It should 
	* not be used in realtime. 
	*
	* @param iX <code>int</code> denoting which pixel on the X axis we are targeting.
	* @param iY <code>int</code> denoting which pixel on the y axis we are targeting.
	* @return <code>MinuetoColor</code> denoting the color of the specified pixel.
	**/
	/*public MinuetoColor getPixel(int iX, int iY) {
	  	
		// You should really implement this
	  		return null;
}*/

	/**
	 * Draw another <code>MinuetoImage</code> on this <code>MinuetoImage</code> (thus compositing them).
	 *
	 * @param mimImage <code>MinuetoImage</code> that we will draw on our image.
	 * @param iX <code>int</code> denoting where we should start drawing on the x axis.
	 * @param iY <code>int</code> denothing where we should start drawing on the y axis.
	 **/
	public void draw(MinuetoImage mimImage, int iX, int iY) {

		if (mimImage == null) new NullPointerException();
		
		this.g2dGraphics.drawImage(mimImage.getImage(),iX,iY,null);
		//this.g2dGraphics.dispose();
	}

	/**
	 * Draw a line of the specified color from the start point to the end point.
	 *
	 * @param mcoColor <code>MinuetoColor</code> denoting the color of the line.
	 * @param iXStart <code>int</code> denoting the X value of the start point.
	 * @param iYStart <code>int</code> denoting the Y value of the start point.
	 * @param iXStop <code>int</code> denoting the X value of the end point.
	 * @param iYStop <code>int</code> denoting the Y value of the end point.
	 * @throws <code>MinuetoOutOfBoundException</code> if the target coordinates are not
	 * valid.
	 **/

	public void drawLine(MinuetoColor mcoColor, int iXStart, int iYStart, int iXStop, int iYStop) {

		if (mcoColor == null) new NullPointerException();

		int iWidth = this.iWidth;
		int iHeight = this.iHeight;

		/* Bound check */
		if (( iXStart < 0 ) || ( iXStart >= iWidth))
			throw new MinuetoOutOfBoundException(iXStart, 0, iWidth);
		//if (( iXStop < 0 ) || ( iXStop >= iWidth))
		//	throw new MinuetoOutOfBoundException(iXStop, 0, iWidth);
		if (( iYStart < 0 ) || ( iYStart >= iHeight))
			throw new MinuetoOutOfBoundException(iYStart, 0, iHeight);
		//if (( iYStop < 0 ) || ( iYStop >= iHeight))
		//	throw new MinuetoOutOfBoundException(iYStop, 0, iHeight);

		this.g2dGraphics.setColor(mcoColor.getAWTColor());
		this.g2dGraphics.drawLine(iXStart,iYStart,iXStop,iYStop);
	}

	/**
	 * Return the width of the image.
	 *
	 * @return	<code>int</code> denoting the width of the image.
	 **/
	public int getWidth() {

		return this.iWidth;
	}

	/**
	 * Return the height of the image.
	 *
	 * @return	<code>int</code> denoting the height of the image.
	 **/
	public int getHeight() {

		return this.iHeight;
	}

	/**
	 * Return the buffered image (content) of this MinuetoImage. This should
	 * only be called by methods in the package.
	 *
	 * @return <code>BufferedImage</code> of the <code>MinuetoImage</code>.
	 **/
	protected BufferedImage getImage() {

		if (this.bAccelerated == false) this.accelerateBuffer();

		return this.bufImage;
	}
	
	protected boolean accelerateBuffer() {
			
		Graphics2D g2dGraphics;
				
		BufferedImage bufImage;
		
		if (this.bAccelerated == true) return true;
		
		bufImage = ImageTools.createAcceleratedImage(
			this.bufImage.getWidth(),
			this.bufImage.getHeight());
			
		if (bufImage == null) return false;
						
		g2dGraphics = bufImage.createGraphics();
				
		/* Copy image */
		g2dGraphics.drawImage(this.bufImage ,0,0,null);
		g2dGraphics.dispose();
		
		this.bufImage = bufImage;
		this.g2dGraphics = this.bufImage.createGraphics();		
		
		this.bAccelerated = true;
		return true;
	}	

	/**
	 * Saves the content of the MinuetoImage to a PNG file. Since this method
	 * requires File I/O, it is somewhat slow. However, this method can be
	 * uses as a debugging tool.
	 *
	 * @param strFilename String indicating the name of the PNG file.
	 *
	 * @throws <code>MinuetoFileException</code> if it cannot write the file.
	 **/
	public void save(String strFilename) throws MinuetoFileException {

		if (strFilename == null) new NullPointerException();

		try {
			File file = new File(strFilename);
			ImageIO.write(this.bufImage, "png", file);
		} catch ( IOException ioe) {
			throw new MinuetoFileException("Cannot write to \"" + strFilename + "\" .");
		}
	}
	
	/**
	 * Creates and returns a copy of this MinuetoImage. The returned MinuetoImage
	 * is independant from this MinuetoImage. 
	 **/
	public Object clone() {
	
		MinuetoImage mimClone = new MinuetoImage(this.iWidth, this.iHeight);
		mimClone.draw(this, 0, 0);
		
		return mimClone;
	}

}


