/*
 * @(#)MinuetoText.java
 *
 * Minueto - The Game Development Framework 
 * Copyright (c) 2004 McGill University
 * 3480 University Street, Montreal, Quebec H3A 2A7
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.minueto.image;

import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;

import org.minueto.MinuetoColor;

/**
 * The <code>MinuetoText</code> class allows you to build an image of a string of text. 
 * You can draw that image on the screen or over another image.
 * <p>
 * Since <code>MinuetoText</code> extends <code>MinuetoImage</code>, all the important image
 * manipulation fonctionnality (such as draw, drop, scale, etc) are available.
 *
 * @author	Alexandre Denault
 * @version 1.0
 * @since 	Minueto 0.4
 * @see 		MinuetoImage
 **/
public class MinuetoText extends MinuetoImage {

	/**
	 * Builds a <code>MinuetoText</code> image of a string of the specified 
	 * color and font.
	 *
	 * @param strText <code>String</code> denoting the text to be used to create the image.
	 * @param mcoColor <code>MinuetoColor</code> denoting the color of the string.
	 * @param mfoFont <code>MinuetoFont</code> describing the font used to draw the string.
	 **/	
   public  MinuetoText(String strText, MinuetoFont mfoFont, MinuetoColor mcoColor) {
	
		/* When subclassing to a class with multiple constructor,it's
		* important to specify which constructor to use. */
		super();
		
		this.setUpText(strText, mfoFont, mcoColor, false);
   }
	
	/**
	 * Builds a <code>MinuetoText</code> image of a string of the specified 
	 * color and font. The string can also anti-aliased. Anti-aliasing tries to
	 * improve the visibility of a font by introducing variation in the colors
	 * used to draw the text. This gives great result for large fonts, but should
	 * be avoided with small fonts.
	 *
	 * @since 0.45
	 * @param strText <code>String</code> denoting the text to be used to create the image.
	 * @param mcoColor <code>MinuetoColor</code> denoting the color of the string.
	 * @param mfoFont <code>MinuetoFont</code> describing the font used to draw the string.
	 ** @param bAntiAliased <code>boolean</code> indicating if the text should be anti-aliased.
	 **/
	public MinuetoText(String strText, MinuetoFont mfoFont, MinuetoColor mcoColor, boolean bAntiAliased) {
	
		/* When subclassing to a class with multiple constructor,it's
		* important to specify which constructor to use. */
		super();
		
		this.setUpText(strText, mfoFont, mcoColor, bAntiAliased);
	}
	
	/**
	 * Builds the actual MinuetoText objects. The constructors are wrappers for this function.
	 */
	public void setUpText(String strText, MinuetoFont mfoFont, MinuetoColor mcoColor, boolean bAntiAliased) {
	
		/* The hardest part of build the image with the text string is 
		 * figuring out the size of the string itself. **/
		
		if (strText == null) new NullPointerException();
		if (mfoFont == null) new NullPointerException();
		if (mcoColor == null) new NullPointerException();
		
		BufferedImage bufImage;
		Graphics2D g2dGraphics;
		FontMetrics ftmMetrics;
						
		int iWidth;		/* Width of our image */
		int iHeight;	/* Height of our image */
   		
		/* To calculate the size of the string, we need to get information
		 * about the screen (since the string size depends on the DPI 
		 * resolution of the monitor). To get this information, we create
		 * a temporary 1x1 accelerated image and get the font metric
		 * information from the graphic 2D image. */
		bufImage = ImageTools.createImage( 1, 1 );
		g2dGraphics = bufImage.createGraphics();
	
		/* Before getting the font metric, we need to set the font on the
		 * grpahic 2D object. */
   		g2dGraphics.setFont(mfoFont.getFont());
			
			/* Enable or disable antialiasing. */
   		if (bAntiAliased == true) {
   			g2dGraphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                             RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
   		} else {
				g2dGraphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                             RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
			}
   		
		/* Ah! now we have the font metrics! */
   		ftmMetrics = g2dGraphics.getFontMetrics();   		
   		   		
		/* It might seem simple, but figuring out that these 2 functions exist
		 * was insanely hard. The Java font engine can be quite complicated. */
   		iHeight = ftmMetrics.getHeight();
   		iWidth = ftmMetrics.stringWidth(strText);
   		
   		/* Since we should never create an image of width 0. */
   		if (iWidth == 0) { iWidth = 1; }
   		
		/* So now we know the proper size of the image. */
		bufImage = ImageTools.createImage( iWidth, iHeight );
   		g2dGraphics = bufImage.createGraphics();
	
		/* Set the font and the color. */
   		g2dGraphics.setFont(mfoFont.getFont());
   		g2dGraphics.setColor(mcoColor.getAWTColor());
   		
   		/* Enable or disable antialiasing. */
   		if (bAntiAliased == true) {
   			g2dGraphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                             RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
   		} else {
				g2dGraphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                             RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
			}
   		
   	   	
		/* Draw the string on the buffer. */
		g2dGraphics.drawString(strText, 0, ftmMetrics.getAscent());
   	   	
		/* Set up the MinuetoImage */
   	super.setUpImage(bufImage);
	}
}
