/*
 * @(#)MinuetoFrame.java 
 *
 * Minueto - The Game Development Framework 
 * Copyright (c) 2004 McGill University
 * 3480 University Street, Montreal, Quebec H3A 2A7
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.minueto.window;

import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Point;

import javax.swing.JFrame;
import javax.swing.JPanel;

import org.minueto.MinuetoOutOfBoundException;
import org.minueto.image.ImageTools;
import org.minueto.image.MinuetoImage;

/**
 * The <code>MinuetoPanel</code> class allows you to create a window (not fullscreen). 
 * This window is your main drawing surface (your canvas). This class is a subclass of 
 * <code>MinuetoWindow</code>. As such, all the drawing functionnality is found in the
 * super class.
 *<p>
 * When a <code>MinuetoWindow</code> is created, it is invisible. The <code>setVisible</code>
 * method must be used toshow the window (make it visible).
 * <p>
 * The size of the <code>MinuetoWindow</code> must be given to the construtor.
 * Once a <code>MinuetoWindow</code> is created, it cannot be resized. To display
 * a MinuetoWindow of different size, the current window must be closed using the 
 * <code>close</code> method and a new <code>MinuetoWindow</code> must be created. 
 * <p>
 * Minueto draws to the screen using a double-buffering strategy. In other words, 
 * there are two drawing surfaces : your window itself and an off screen surface.
 * All your drawing occurs on the off screen surface. The content of the off 
 * screen buffer is drawn to the screen when the <code>render</code> method is invoked.
 * <p>
 * 
 * @author Alexandre Denault
 * @version 1.0
 * @since  Minueto 0.4
 * @see   org.minueto.window.MinuetoWindow
 * @see   org.minueto.image.MinuetoImage
 */
public class MinuetoFrame extends MinuetoWindow {
	
	private JFrame		  jfrFrame;
	private JPanel        jpaPanel;
	private Canvas        canCanvas;	
	
	/**
	 * Creates a <code>MinuetoWindow</code> of the specified size. The second parameter 
	 * determines if the window should be bordeless or not. 
	 *  
	 * @param iWidth <code>int</code> denoting the desired width of the window.
	 * @param iHeight <code>int</code> denoting the desired height of the window.
	 * @param border <code>boolean</code> indicating if the window should be have borders or not. 	 
	 *
	 * @throws <code>MinuetoZeroNegativeException</code> if the window size is invalid.
	 */
	public MinuetoFrame(int iWidth, int iHeight, boolean border) {
		
		super(iWidth, iHeight);
		
		GraphicsDevice			graphicDevice;
		GraphicsEnvironment 	graphicEnvironment;
		
		this.visible = false;
		
		graphicEnvironment = GraphicsEnvironment.getLocalGraphicsEnvironment();
		
		/* Minueto uses the default graphic environment. We can assume this because
		 * Minueto does not yet support dual display configuration. */
		graphicDevice = graphicEnvironment.getDefaultScreenDevice();
		graphicConfiguration = graphicDevice.getDefaultConfiguration();
		
		/* Our window is actually a JFrame that we use as a painting surface.*/
		this.jfrFrame = new JFrame(graphicConfiguration);
		
		this.jfrFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.jfrFrame.setTitle("Minueto");
		this.jfrFrame.setResizable(false);
		this.jfrFrame.setFocusTraversalKeysEnabled(false);		
		this.jfrFrame.setUndecorated(!border);
		
		this.jfrFrame.addKeyListener(this.getKeyListener());
		this.jfrFrame.addMouseListener(this.getMouseListener());
		this.jfrFrame.addMouseMotionListener(this.getMouseMotionListener());
		this.jfrFrame.addWindowListener(this.getWindowListener());
		
		this.jpaPanel = (JPanel)this.jfrFrame.getContentPane();
		
		this.jpaPanel.setLayout(null);
		this.jpaPanel.setPreferredSize(new Dimension(iWidth, iHeight));
		this.jpaPanel.setFocusTraversalKeysEnabled(false);
		
		this.jpaPanel.addKeyListener(this.getKeyListener());
		this.jpaPanel.addMouseListener(this.getMouseListener());
		this.jpaPanel.addMouseMotionListener(this.getMouseMotionListener());
		
		this.canCanvas = new Canvas(graphicConfiguration);
		this.canCanvas.setIgnoreRepaint(true);
		this.canCanvas.setBounds( 0, 0, iWidth, iHeight);
		this.canCanvas.setFocusTraversalKeysEnabled(false);
		
		this.canCanvas.addKeyListener(this.getKeyListener());
		this.canCanvas.addMouseListener(this.getMouseListener());
		this.canCanvas.addMouseMotionListener(this.getMouseMotionListener());
		
		this.jpaPanel.add(this.canCanvas);
		
		ImageTools.setGraphicConfiguration(graphicConfiguration);
		
		WindowManager.getInstance().register(this, false);
	}
	
	public synchronized void setVisible(boolean bValue) {
		
		/* We record if the screen is visible or not. This is important to
		 * remember because we need to know the visibility when resize if 
		 * called. */
		this.visible = bValue;
		
		/* Set the window to the proper size. */
		//this.jfrFrame.setSize(this.iWidth, this.iHeight);
		
		if (bValue == false) {
			
			/* Hides the window. */
			this.jfrFrame.setVisible(false);
			
		} else {
			
			/* Show the window */
			this.jfrFrame.setVisible(true);
			
			this.jfrFrame.pack();
			
			/* We need to initialized the double-buffer mode the first time we show the window */
			if (grpGraphicsPointer == null) {
				
				/* Initialize the bufferstrategy for the frame, thus allowing us
				 * to double buffer the screen. 
				 * Note: The buffer strategy can only be created once the window
				 * is visible.*/
				this.canCanvas.createBufferStrategy(2);
				this.busBufferStrategy = this.canCanvas.getBufferStrategy();
				
				/* Create the Graphics Pointer we will need to draw on the screen.
				 * This pointer needs to be recreated everytime we render (draw) the
				 * screen. */
				this.grpGraphicsPointer = (Graphics2D)this.busBufferStrategy.getDrawGraphics();	
			}			
		}		
	}
	
	public synchronized void close() {
		//this.render();
		
		this.setVisible(false);
		
		this.grpGraphicsPointer.dispose();
		this.grpGraphicsPointer = null;	
		
		this.jfrFrame.dispose();
		this.jfrFrame = null;	
		
		this.getKeyListener().clear();
		this.getMouseListener().clear();
		this.getMouseMotionListener().clear();
		this.getWindowListener().clear();
		
		WindowManager.getInstance().unregister(this);
		
	}
	
	public synchronized void setTitle(String strTitle)
	{
		
		if (strTitle == null)
			new NullPointerException();
		if (this.visible == false)
			throw new MinuetoWindowInvalidStateException("Window is invisible");
		
		this.jfrFrame.setTitle(strTitle);
	}
	
	/**
	 * Returns the width of the current window. Please note that
	 * the width returned is the actual width of the window, not
	 * the requested width. This is important to take into acccount because
	 * operating systems such as Window add a thin border around every window.
	 * To properly center a window, the actual width of the window must
	 * be known.
	 *
	 * @return <code>int</code> denoting the real width of the window.
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden (invisible) or has been closed.
	 */
	public synchronized int getWindowWidth()
	{
		
		return this.jfrFrame.getWidth();
	}
	
	/**
	 * Returns the height of the current window. Please note that
	 * the height returned is the actual height of the window, not
	 * the requested height.  This is important to take into acccount because
	 * operating systems such as Window add a thin border around every window.
	 * To properly center a window, the actual width of the window must
	 * be known.
	 *
	 * @return <code>int</code> denoting the real height of the window.
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden (invisible) or has been closed.  
	 */
	public synchronized int getWindowHeight()
	{
		
		return this.jfrFrame.getHeight();
	}
	
	/**
	 * Returns the current X position of this window on the desktop. A value of
	 * 0 is returned if the window is in fullscreen mode.
	 */
	public synchronized int getWindowPositionX(){
		
		return this.jfrFrame.getX();
	}
	
	/**
	 * Returns the current Y position of this window. A value of 0 is returned
	 * if the window is in fullscreen mode.
	 */
	public synchronized int getWindowPositionY() {
		
		return this.jfrFrame.getY();
	}
	
	/**
	 * Moves the window to the desired location. This method is ignored if Minueto is
	 * fullscreen mode.
	 *
	 * @param iX <code>int</code> denoting the X position where the window should be moved.
	 * @param iY <code>int</code> denoting the Y position where the window should be moved.
	 *
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden (invisible) or has been closed.
	 */
	public synchronized void setWindowPosition(int iX, int iY) {
		
		if (this.visible == false)
			throw new MinuetoWindowInvalidStateException("Window is invisible");
		
		if ( iX < 0 )
			throw new MinuetoOutOfBoundException(iX, 0, 999999);
		if ( iY < 0 )
			throw new MinuetoOutOfBoundException(iY, 0, 999999);
		
		this.jfrFrame.setLocation(iX, iY);
	}
	
	/**
	 * By default, when a MinuetoWindow is closed by the operating system (click on 
	 * the X in the top right corner for example), the whole application is closed.
	 * This method allows you to change this behavior.
	 * 
	 * @param value <code>boolean</code> denoting if the whole application should be closed.
	 */
	public synchronized void exitOnClose(boolean value) {
		
		if(value) {
			this.jfrFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		} else {
			this.jfrFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		}
	}
	
	
	public synchronized void setCursorVisible(boolean bVisible) {
		
		// Code to hide the cursor taken from
		// http://www.rgagnon.com/javadetails/java-0440.html
		
		if(bVisible == false) {
			
			this.jfrFrame.setCursor(this.jfrFrame.getToolkit().createCustomCursor(										
					ImageTools.getBufferedImage(new MinuetoImage(3,3)),
					new Point(0, 0), "null"));
			
		} else {
			
			this.jfrFrame.setCursor(null);
		}
	}
	
}
