/*
 * @(#)MinuetoWindow.java 
 *
 * Minueto - The Game Development Framework 
 * Copyright (c) 2004 McGill University
 * 3480 University Street, Montreal, Quebec H3A 2A7
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.minueto.window;

import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.Toolkit;
import java.awt.image.BufferStrategy;

import org.minueto.MinuetoColor;
import org.minueto.MinuetoEventQueue;
import org.minueto.MinuetoOutOfBoundException;
import org.minueto.MinuetoStopWatch;
import org.minueto.handlers.MinuetoKeyListener;
import org.minueto.handlers.MinuetoKeyboardHandler;
import org.minueto.handlers.MinuetoMouseHandler;
import org.minueto.handlers.MinuetoMouseListener;
import org.minueto.handlers.MinuetoMouseMotionListener;
import org.minueto.handlers.MinuetoWindowHandler;
import org.minueto.handlers.MinuetoWindowListener;
import org.minueto.image.ImageTools;
import org.minueto.image.MinuetoImage;

/**
 * The <code>MinuetoWindow</code> abstract defines the methods available to all 
 * classes that create a Minueto window (or drawing surface). 
 *  
 * @author 	Alexandre Denault
 * @version 1.0
 * @since  	Minueto 1.0
 * @see   	org.minueto.window.MinuetoFrame
 * @see		org.minueto.window.MinuetoFullscreen
 * @see 	org.minueto.window.MinuetoPanel
 */
public abstract class MinuetoWindow {

	/** Reference to the drawing object. This needs to be updated after
	 *  a page flip. */
	protected Graphics2D       	grpGraphicsPointer;
	/** Allows us to do double buffering. */
	protected BufferStrategy	busBufferStrategy;
	
	protected GraphicsConfiguration	graphicConfiguration;
	
	/** Width of the screen */
	private int         iWidth;
	/** Height of the screen */
	private int         iHeight;
	
	/** Boolean value indicating if the screen should be visible. */
	protected boolean        	visible;

	/** This JFrame is our drawing window/surface. */
	//protected JFrame      		jfrFrame;

	private double        		dTimeBetweenFrames;
	private double        		dMaxFrameRate;
	private double        		dCurrentFrameRate;
	private int         		iFrameCount;

	private MinuetoStopWatch     mswMaxFrameRate;
	private MinuetoStopWatch     mswCurrentFrameRate;
	
	private MinuetoKeyListener			keyListener;
	private MinuetoMouseListener 		mouseListener;
	private MinuetoMouseMotionListener 	mouseMotionListener;
	private MinuetoWindowListener		windowListener;
	
	protected MinuetoWindow(int iWidth, int iHeight) {
		
		/* Save the new attributes of the window. */
		this.iWidth = iWidth;
		this.iHeight = iHeight;
		
		this.dTimeBetweenFrames = 0;
		this.dMaxFrameRate = 0;
		this.dCurrentFrameRate = 0;
		this.iFrameCount = 0;

		this.mswCurrentFrameRate = new MinuetoStopWatch();
		this.mswMaxFrameRate = new MinuetoStopWatch();

		this.mswCurrentFrameRate.start();
		this.mswMaxFrameRate.start();
		
		this.keyListener = new MinuetoKeyListener();
		this.mouseListener = new MinuetoMouseListener();
		this.mouseMotionListener = new MinuetoMouseMotionListener();
		this.windowListener = new MinuetoWindowListener();
	}

	/**
	 * Draws an image on the backbuffer of our window.
	 *
	 * @param mimImage <code>MinuetoImage</code> drawn on the back buffer.
	 * @param iX <code>int</code> denoting the X position where the image should be drawn.
	 * @param iY <code>int</code> denoting the Y position where the image should be drawn.
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden (invisible) or has been closed.  
	 */
	public synchronized void draw(MinuetoImage mimImage, int iX, int iY)
	{
		if (mimImage == null)
			throw new NullPointerException();		
		if (this.visible == false)
			throw new MinuetoWindowInvalidStateException("Window is invisible");

		/* Uses the graphicpointer to draw the bufferedimage on the backbuffer. */
		this.grpGraphicsPointer.drawImage(
				ImageTools.getBufferedImage(mimImage), iX, iY, null);
	}

	/**
	 * Draw a line of the specified color from the start point to the end point.
	 *
	 * @param mcoColor <code>MinuetoColor</code> denoting the color of the line.
	 * @param iXStart <code>int</code> denoting the X value of the start point.
	 * @param iYStart <code>int</code> denoting the Y value of the start point.
	 * @param iXStop <code>int</code> denoting the X value of the end point.
	 * @param iYStop <code>int</code> denoting the Y value of the end point.
	 * @throws <code>MinuetoOutOfBoundException</code> if the target coordinates are not
	 * valid.
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden (invisible) or has been closed.  
	 **/
	public synchronized void drawLine(MinuetoColor mcoColor, int iXStart, int iYStart, int iXStop, int iYStop)
	{

		if (mcoColor == null)
			new NullPointerException();
		if (this.visible == false)
			throw new MinuetoWindowInvalidStateException("Window is invisible");


		int iWidth = this.iWidth;
		int iHeight = this.iHeight;

		if (( iXStart < 0 ) || ( iXStart >= iWidth))
			throw new MinuetoOutOfBoundException(iXStart, 0, iWidth);
		if (( iXStop < 0 ) || ( iXStop >= iWidth))
			throw new MinuetoOutOfBoundException(iXStop, 0, iWidth);
		if (( iYStart < 0 ) || ( iYStart >= iHeight))
			throw new MinuetoOutOfBoundException(iYStart, 0, iHeight);
		if (( iYStop < 0 ) || ( iYStop >= iHeight))
			throw new MinuetoOutOfBoundException(iYStop, 0, iHeight);

		this.grpGraphicsPointer.setColor(mcoColor.getAWTColor());

		this.grpGraphicsPointer.drawLine( iXStart,
		                                  iYStart,
		                                  iXStop,
		                                  iYStop);
	}

	/**
	 * Draws the content of the backbuffer onto the window. The content of the 
	 * backbuffer is lost after this call.
	 *
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is 
	 * hidden (invisible) or has been closed.    
	 **/
	public synchronized void render()
	{
        int iTimePause;
		if (this.visible == false)
			throw new MinuetoWindowInvalidStateException("Window is invisible");

		this.iFrameCount++;

		if (this.iFrameCount == 50) {
			this.mswCurrentFrameRate.stop();
			this.dCurrentFrameRate = 50000.0 / this.mswCurrentFrameRate.getTime();
			this.mswCurrentFrameRate.reset();
			this.mswCurrentFrameRate.start();
			this.iFrameCount = 0;
		}			
		
        if (this.dMaxFrameRate > 0) {
            
            iTimePause = (int)(this.dTimeBetweenFrames - this.mswMaxFrameRate.getTime());
            
            while (iTimePause > 10) {
                
				try {
                    Thread.sleep(iTimePause/2);
                } catch (InterruptedException ie) {
                    
                }
                
                iTimePause = (int)(this.dTimeBetweenFrames - this.mswMaxFrameRate.getTime());
			}
            
            while (this.mswMaxFrameRate.getTime() < this.dTimeBetweenFrames) {
                
                Thread.yield();                
			}
        }

		/* This should allow our fonts to be nicer, I think. */
		//this.grpGraphicsPointer.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

		/* Releases the graphic pointer. We wont be needing it anymore. */
		this.grpGraphicsPointer.dispose();

		/* Copies the backbuffer to the frontbuffer. */
		this.busBufferStrategy.show();

		/* Sync the display on some systems. (on Linux, this fixes event queue
		 * problems). */
		Toolkit.getDefaultToolkit().sync();

		/* Get a new graphicpointer */
		this.grpGraphicsPointer = (Graphics2D)this.busBufferStrategy.getDrawGraphics();

		this.mswMaxFrameRate.reset();
		this.mswMaxFrameRate.start();
	}

	/**
	 * Returns the width of the area your are allowed to draw in. This value
	 * is identical to the width specified to this constructor.
	 *
	 * @since 0.4.7
	 * @return <code>int</code> denoting the draw width of the window.
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden (invisible) or has been closed.  
	 */
	public synchronized int getWidth() {

		return this.iWidth;
	}

	/**
	 * Returns the height of the area your are allowed to draw in. This value
	 * is identical to the height specified to this constructor.
	 *
	 * @since 0.4.7
	 * @return <code>int</code> denoting the draw height of the window.
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden (invisible) or has been closed.  
	 */
	public synchronized int getHeight() {

		return this.iHeight;
	}
	
	/**
	 * Erases the content of the backbuffer. The color of a cleared buffer will depend on the 
	 * JVM.
	 *
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden 
	 *    (invisible) or has been closed.  
	 **/
	public synchronized void clear()
	{

		if (this.visible == false)
			throw new MinuetoWindowInvalidStateException("Window is invisible");

		/* Clear the whole screen. */
		this.grpGraphicsPointer.clearRect(0,0,this.iWidth,this.iHeight);
	}

	/**
	 * Erases the content of the backbuffer. The color of a cleared buffer is specified
	 * as a parameter.
	 * 
	 * @param mcoColor <code>MinuetoColor</code> of the cleared buffer.
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden 
	 *    (invisible) or has been closed.  
	 **/
	public synchronized void clear(MinuetoColor mcoColor)
	{

		if (mcoColor == null)
			new NullPointerException();
		if (this.visible == false)
			throw new MinuetoWindowInvalidStateException("Window is invisible");

		this.grpGraphicsPointer.setColor(mcoColor.getAWTColor());

		/* Clear the whole screen. */
		this.grpGraphicsPointer.fillRect(0,0,this.iWidth,this.iHeight);
	}	

	/**
	 * Returns the number of frame renders per second. This is value 
	 * is updated every 50 frames.
	 *
	 * @since 0.6
	 *
	 * @return <code>int</code> denoting the number of frames drawn per second.
	 **/
	public synchronized double getFrameRate()
	{

		return this.dCurrentFrameRate;
	}

	/**
	 * Caps the number of frames which can be rendered (display) per second. 
	 * The efficienty of the cap depends on the accuracy of the system timer.
	 * Calling this method with a value of 0 will disable the cap.
	 *
	 * @since 0.6
	 *
	 * @param dValue <code>double</code> denoting the desired number of frames to
	 * be displayed per second.
	 **/
	public synchronized void setMaxFrameRate(double dValue)
	{

		if (dValue < 1.0)
			dValue = 0;

		this.dMaxFrameRate = dValue;
		this.dTimeBetweenFrames = 1000.0 / dValue;
	}


	/**
	 * Saves the content of the <code>MinuetoWindow</code> to a PNG file. This
	 * operation is very slow since the content of the screen must be retrieved
	 * from the video(card) memory.
	 *
	 * @param strFilename <code>String</code> indicating the name of the target PNG file.
	 *
	 * @throws <code>MinuetoFileException</code> if it cannot write the file.
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden (invisible) or has been closed.
	 **/
	/*public synchronized void save(String strFilename)
	throws MinuetoFileException
	{

		if (strFilename == null)
			new NullPointerException();
		if (this.visible == false)
			throw new MinuetoWindowInvalidStateException("Window is invisible");

		this.getScreenshot().save(strFilename);
	}*/

	/**
	 * Returns a <code>MinuetoImage</code> containing the image of the <code>MinuetoWindow</code>.
	 * This operation is very slow since the content of the screen must be retrieved
	 * from the video(card) memory.
	 *
	 * @return <code>MinuetoImage</code> containing the image currently displayed on-screen.
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden (invisible) or has been closed.  
	 **/
	/*public synchronized MinuetoImage getScreenshot() {

		BufferedImage bufUnacceleratedImage;
		Rectangle recAreaToCapture;

		if (this.visible == false)
			throw new MinuetoWindowInvalidStateException("Window is invisible");

		recAreaToCapture = new Rectangle( this.getWindowPositionX(),
		                                  this.getWindowPositionY(),
		                                  this.iWidth,
		                                  this.iHeight);

		try {

			bufUnacceleratedImage = new Robot().createScreenCapture(recAreaToCapture);

		} catch (Exception e) {
			return null;
		}

		return new MinuetoImage(bufUnacceleratedImage);
		return null;
	}*/
	
	/**
	 * Assign a keyboard handler to the <code>MinuetoWindow</code> to process 
	 * keyboard input. An unlimited number of keyboard handlers can be registered
	 * with a window, but too many handlers may slow the application (because
	 * of the large number of events produced). 	 
	 *
	 * @param handler <code>MinuetoKeyboardHandler</code> to register with the 
	 * <code>MinuetoEventQueue</code>.
	 * @param queue <code>MinuetoEventQueue</code> that will store events..
	 * @since Minueto 0.8
	 **/
	public void registerKeyboardHandler(MinuetoKeyboardHandler handler, 
			MinuetoEventQueue queue) {
		
		this.keyListener.register(handler, queue);
	}

	/**
	 * Assign a mouse handler to the <code>MinuetoWindow</code> to process 
	 * mouse input. An unlimited number of keyboard handlers can be registered
	 * with a window, but too many handlers may slow the application (because
	 * of the large number of events produced). 
	 *
	 * @param handler <code>MinuetoMouseHandler</code> to register with the 
	 * MinuetoWindow.
	 * @param queue <code>MinuetoEventQueue</code> that will store events.
	 * @since Minueto 0.8
	 **/
	public void registerMouseHandler(MinuetoMouseHandler handler, 
			MinuetoEventQueue queue) {
		
		this.mouseListener.register(handler, queue);
		this.mouseMotionListener.register(handler, queue);
	}

	/**
	 * Assign a mouse handler to the <code>MinuetoWindow</code> to process 
	 * mouse input. An unlimited number of keyboard handlers can be registered
	 * with a window, but too many handlers may slow the application (because
	 * of the large number of events produced). 
	 *
	 * @param handler <code>MinuetoMouseHandler</code> to register with the 
	 * MinuetoWindow.
	 * @param queue <code>MinuetoEventQueue</code> that will store events.
	 * @since Minueto 0.8
	 **/
	public void registerWindowHandler(MinuetoWindowHandler handler, 
			MinuetoEventQueue queue) {

		this.windowListener.register(handler, queue);
	}
	
	/**
	 * Removes a keyboard handler from the <code>MinuetoWindow</code>. Note that
	 * both the correct handler and queue must be given to unregister the handler.
	 *
	 * @param handler <code>MinuetoKeyboardHandler</code> to unregister with the 
	 * MinuetoWindow.
	 * @param queue <code>MinuetoEventQueue</code> that stored the events.
	 * @since Minueto 1.0
	 **/
	public void unregisterKeyboardHandler(MinuetoKeyboardHandler handler, 
			MinuetoEventQueue queue) {
		
		this.keyListener.unregister(handler, queue);
	}

	/**
	 * Removes a mouse handler from the <code>MinuetoWindow</code>. Note that
	 * both the correct handler and queue must be given to unregister the handler.
	 *
	 * @param handler <code>MinuetoMouseHandler</code> to unregister with the 
	 * MinuetoWindow.
	 * @param queue <code>MinuetoEventQueue</code> that stored the events.
	 * @since Minueto 1.0
	 **/
	public void unregisterMouseHandler(MinuetoMouseHandler handler, 
			MinuetoEventQueue queue) {
		
		this.mouseListener.unregister(handler, queue);
		this.mouseMotionListener.unregister(handler, queue);
	}

	/**
	 * Removes a window handler from the <code>MinuetoWindow</code>. Note that
	 * both the correct handler and queue must be given to unregister the handler.
	 *
	 * @param handler <code>MinuetoWindowHandler</code> to unregister with the 
	 * MinuetoWindow.
	 * @param queue <code>MinuetoEventQueue</code> that stored the events.
	 * @since Minueto 1.0
	 **/
	public void unregisterWindowHandler(MinuetoWindowHandler handler, 
			MinuetoEventQueue queue) {

		this.windowListener.unregister(handler, queue);
	}
	
	/**
	 * Close the window and free any ressource still in use by the window.
	 * Once executed, this window can no longuer be used. Any
	 * further invocation of a method from this object will result in a 
	 * <code>MinuetoWindowInvalidStateException</code>.
	 */
	public abstract void close();
	
	/**
	 * Shows or hides the mouse cursor.
	 * 
	 * @param value <code>boolean</code> denoting if the window should be visible or not.
	 * @since 0.4.6
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden 
	 *    (invisible) or has been closed.
	 */
	public abstract void setCursorVisible(boolean value);
	
	/**
	 * Change the title of the window.
	 * 
	 * @param title <code>String</code> identifying the desired title of the screen.
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden 
	 *    (invisible) or has been closed.
	 */
	public abstract void setTitle(String title);
	
	/**
	 * Shows or hides the MinuetoWindow.
	 * 
	 * @param value <code>boolean</code> denoting if the window should be visible or not.
	 * @throws <code>MinuetoWindowInvalidStateException</code> if this window is hidden 
	 *    (invisible) or has been closed.
	 */
	public abstract void setVisible(boolean value);
	
	
	protected MinuetoKeyListener getKeyListener() {
		
		return keyListener;
	}


	protected MinuetoMouseListener getMouseListener() {
		
		return mouseListener;
	}


	protected MinuetoMouseMotionListener getMouseMotionListener() {
		
		return mouseMotionListener;
	}


	protected MinuetoWindowListener getWindowListener() {
		
		return windowListener;
	}
	
	



}